"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from typing import TYPE_CHECKING
from importlib import import_module
import builtins
import sys

if TYPE_CHECKING:
    from .anthropiccontent import AnthropicContent, AnthropicContentTypedDict
    from .anthropiccontentblockdelta import (
        AnthropicContentBlockDelta,
        AnthropicContentBlockDeltaDelta,
        AnthropicContentBlockDeltaDeltaTypedDict,
        AnthropicContentBlockDeltaType,
        AnthropicContentBlockDeltaTypedDict,
    )
    from .anthropiccontentblockstart import (
        AnthropicContentBlockStart,
        AnthropicContentBlockStartInput,
        AnthropicContentBlockStartInputTypedDict,
        AnthropicContentBlockStartType,
        AnthropicContentBlockStartTypedDict,
        ContentBlock,
        ContentBlockTypedDict,
    )
    from .anthropiccontentblockstop import (
        AnthropicContentBlockStop,
        AnthropicContentBlockStopContentBlock,
        AnthropicContentBlockStopContentBlockTypedDict,
        AnthropicContentBlockStopTypedDict,
    )
    from .anthropicdocumentcontent import (
        AnthropicDocumentContent,
        AnthropicDocumentContentTypedDict,
    )
    from .anthropicdocumentsource import (
        AnthropicDocumentSource,
        AnthropicDocumentSourceTypedDict,
    )
    from .anthropicdocumentsourcebase64 import (
        AnthropicDocumentSourceBase64,
        AnthropicDocumentSourceBase64MediaType,
        AnthropicDocumentSourceBase64TypedDict,
    )
    from .anthropicdocumentsourceurl import (
        AnthropicDocumentSourceURL,
        AnthropicDocumentSourceURLTypedDict,
    )
    from .anthropicimagecontent import (
        AnthropicImageContent,
        AnthropicImageContentTypedDict,
    )
    from .anthropicimagesource import (
        AnthropicImageSource,
        AnthropicImageSourceTypedDict,
    )
    from .anthropicimagesourcebase64 import (
        AnthropicImageSourceBase64,
        AnthropicImageSourceBase64TypedDict,
        MediaType,
    )
    from .anthropicimagesourceurl import (
        AnthropicImageSourceURL,
        AnthropicImageSourceURLTypedDict,
    )
    from .anthropicinputschema import (
        AnthropicInputSchema,
        AnthropicInputSchemaTypedDict,
        Properties,
        PropertiesTypedDict,
    )
    from .anthropicmessagedelta import (
        AnthropicMessageDelta,
        AnthropicMessageDeltaDelta,
        AnthropicMessageDeltaDeltaTypedDict,
        AnthropicMessageDeltaTypedDict,
        AnthropicMessageDeltaUsage,
        AnthropicMessageDeltaUsageTypedDict,
    )
    from .anthropicmessagemessage import (
        AnthropicMessageMessage,
        AnthropicMessageMessageContent,
        AnthropicMessageMessageContentTypedDict,
        AnthropicMessageMessageRole,
        AnthropicMessageMessageTypedDict,
    )
    from .anthropicmessagerequest import (
        AnthropicMessageRequest,
        AnthropicMessageRequest1,
        AnthropicMessageRequest2,
        AnthropicMessageRequest2TypedDict,
        AnthropicMessageRequestServiceTier,
        AnthropicMessageRequestToolChoice,
        AnthropicMessageRequestToolChoiceTypedDict,
        AnthropicMessageRequestType,
        AnthropicMessageRequestTypedDict,
        Thinking,
        ThinkingTypedDict,
    )
    from .anthropicmessageresponse import (
        AnthropicMessageResponse,
        AnthropicMessageResponseTypedDict,
        AnthropicMessageResponseUsage,
        AnthropicMessageResponseUsageTypedDict,
        StopReason,
    )
    from .anthropicmessagestart import (
        AnthropicMessageStart,
        AnthropicMessageStartContent,
        AnthropicMessageStartContentTypedDict,
        AnthropicMessageStartMessage,
        AnthropicMessageStartMessageTypedDict,
        AnthropicMessageStartTypedDict,
        AnthropicMessageStartUsage,
        AnthropicMessageStartUsageTypedDict,
    )
    from .anthropicping import AnthropicPing, AnthropicPingTypedDict
    from .anthropicstreamevent import (
        AnthropicStreamEvent,
        AnthropicStreamEventTypedDict,
    )
    from .anthropicstreammessagestop import (
        AnthropicStreamMessageStop,
        AnthropicStreamMessageStopTypedDict,
    )
    from .anthropictextcontent import (
        AnthropicTextContent,
        AnthropicTextContentTypedDict,
    )
    from .anthropicthinkingcontent import (
        AnthropicThinkingContent,
        AnthropicThinkingContentTypedDict,
    )
    from .anthropictool import AnthropicTool, AnthropicToolTypedDict
    from .anthropictoolcontent import (
        AnthropicToolContent,
        AnthropicToolContentTypedDict,
    )
    from .anthropictoolresultcontent import (
        AnthropicToolResultContent,
        AnthropicToolResultContentTypedDict,
    )
    from .anthropictoolusecontent import (
        AnthropicToolUseContent,
        AnthropicToolUseContentTypedDict,
    )
    from .audio import Audio, AudioTypedDict, Format, Voice
    from .audiospeechrequest import (
        AudioSpeechRequest,
        AudioSpeechRequestResponseFormat,
        AudioSpeechRequestTypedDict,
        AudioSpeechRequestVoice,
    )
    from .audiotranscriptionrequest import (
        AudioTranscriptionRequest,
        AudioTranscriptionRequestResponseFormat,
        AudioTranscriptionRequestTypedDict,
        File,
        FileTypedDict,
        TimestampGranularities,
    )
    from .audiotranscriptionresponse import (
        AudioTranscriptionResponse,
        AudioTranscriptionResponseTypedDict,
        Segments,
        SegmentsTypedDict,
        Words,
        WordsTypedDict,
    )
    from .audiotranslationrequest import (
        AudioTranslationRequest,
        AudioTranslationRequestFile,
        AudioTranslationRequestFileTypedDict,
        AudioTranslationRequestResponseFormat,
        AudioTranslationRequestTypedDict,
    )
    from .audiotranslationresponse import (
        AudioTranslationResponse,
        AudioTranslationResponseTypedDict,
    )
    from .chatcompletionchoice import (
        ChatCompletionChoice,
        ChatCompletionChoiceTypedDict,
        FinishReason,
        Logprobs,
        LogprobsTypedDict,
    )
    from .chatcompletionrequest import (
        ChatCompletionRequest,
        ChatCompletionRequestFunction,
        ChatCompletionRequestFunctionTypedDict,
        ChatCompletionRequestToolChoice,
        ChatCompletionRequestToolChoiceTypedDict,
        ChatCompletionRequestTypedDict,
        Modalities,
        One,
        ReasoningEffort,
        ServiceTier,
        Stop,
        StopTypedDict,
        Two,
        TwoTypedDict,
    )
    from .chatcompletionresponse import (
        ChatCompletionResponse,
        ChatCompletionResponseTypedDict,
    )
    from .chatcompletionstreamchoice import (
        ChatCompletionStreamChoice,
        ChatCompletionStreamChoiceFinishReason,
        ChatCompletionStreamChoiceLogprobs,
        ChatCompletionStreamChoiceLogprobsTypedDict,
        ChatCompletionStreamChoiceTypedDict,
    )
    from .chatcompletionstreamdelta import (
        ChatCompletionStreamDelta,
        ChatCompletionStreamDeltaRole,
        ChatCompletionStreamDeltaTypedDict,
    )
    from .chatcompletionstreamevent import (
        ChatCompletionStreamEvent,
        ChatCompletionStreamEventTypedDict,
    )
    from .completionchoice import (
        CompletionChoice,
        CompletionChoiceFinishReason,
        CompletionChoiceLogprobs,
        CompletionChoiceLogprobsTypedDict,
        CompletionChoiceTypedDict,
    )
    from .completionrequest import (
        CompletionRequest,
        CompletionRequestStop,
        CompletionRequestStopTypedDict,
        CompletionRequestTypedDict,
    )
    from .completionresponse import CompletionResponse, CompletionResponseTypedDict
    from .completionstreamchoice import (
        CompletionStreamChoice,
        CompletionStreamChoiceFinishReason,
        CompletionStreamChoiceLogprobs,
        CompletionStreamChoiceLogprobsTypedDict,
        CompletionStreamChoiceTypedDict,
    )
    from .completionstreamevent import (
        CompletionStreamEvent,
        CompletionStreamEventTypedDict,
    )
    from .createaudiospeechop import (
        CreateAudioSpeechResponse,
        CreateAudioSpeechResponseTypedDict,
    )
    from .createaudiotranscriptionop import (
        CreateAudioTranscriptionResponse,
        CreateAudioTranscriptionResponseTypedDict,
    )
    from .createaudiotranslationop import (
        CreateAudioTranslationResponse,
        CreateAudioTranslationResponseTypedDict,
    )
    from .createchatcompletionop import (
        CreateChatCompletionResponse,
        CreateChatCompletionResponseBody,
        CreateChatCompletionResponseBodyTypedDict,
        CreateChatCompletionResponseTypedDict,
    )
    from .createcompletionop import (
        CreateCompletionResponse,
        CreateCompletionResponseBody,
        CreateCompletionResponseBodyTypedDict,
        CreateCompletionResponseTypedDict,
    )
    from .createengineembeddingop import (
        CreateEngineEmbeddingRequest,
        CreateEngineEmbeddingRequestTypedDict,
    )
    from .createmessageop import (
        CreateMessageResponse,
        CreateMessageResponseBody,
        CreateMessageResponseBodyTypedDict,
        CreateMessageResponseTypedDict,
    )
    from .createproxyrequestop import (
        CreateProxyRequestRequest,
        CreateProxyRequestRequestTypedDict,
    )
    from .createresponseop import (
        CreateResponseResponse,
        CreateResponseResponseBody,
        CreateResponseResponseBodyTypedDict,
        CreateResponseResponseTypedDict,
    )
    from .editchoice import EditChoice, EditChoiceTypedDict
    from .editrequest import EditRequest, EditRequestTypedDict
    from .editresponse import EditResponse, EditResponseTypedDict
    from .embeddingobject import EmbeddingObject, EmbeddingObjectTypedDict
    from .embeddingrequest import (
        EmbeddingRequest,
        EmbeddingRequestInput,
        EmbeddingRequestInputTypedDict,
        EmbeddingRequestTypedDict,
        EncodingFormat,
    )
    from .embeddingresponse import EmbeddingResponse, EmbeddingResponseTypedDict
    from .engineembeddingrequest import (
        EngineEmbeddingRequest,
        EngineEmbeddingRequestEncodingFormat,
        EngineEmbeddingRequestInput,
        EngineEmbeddingRequestInputTypedDict,
        EngineEmbeddingRequestTypedDict,
    )
    from .functioncall import FunctionCall, FunctionCallTypedDict
    from .editimageop import (
        EditImageResponseBody,
        EditImageResponseBodyTypedDict,
        ImageEditEventData,
        ImageEditEventDataTypedDict,
    )
    from .imageeditrequest import (
        ImageEditBackground,
        ImageEditInputFidelity,
        ImageEditModeration,
        ImageEditOutputFormat,
        ImageEditQuality,
        ImageEditRequest,
        ImageEditRequestTypedDict,
        ImageEditResponseFormat,
        ImageEditSize,
        ImageFile,
        ImageFileTypedDict,
    )
    from .imageeditstreamevent import (
        ImageEditStreamEvent,
        ImageEditStreamEventTypedDict,
    )
    from .imagegenerationrequest import (
        ImageGenerationBackground,
        ImageGenerationModeration,
        ImageGenerationOutputFormat,
        ImageGenerationRequest,
        ImageGenerationRequestResponseFormat,
        ImageGenerationRequestTypedDict,
        Quality,
        Size,
        Style,
    )
    from .imagegenerationresponse import (
        ImageGenerationResponse,
        ImageGenerationResponseTypedDict,
        ImageUsage,
        ImageUsageTypedDict,
    )
    from .imagegenerationstreamevent import (
        ImageGenerationStreamData,
        ImageGenerationStreamDataTypedDict,
        ImageGenerationStreamEvent,
        ImageGenerationStreamEventTypedDict,
    )
    from .createimagegenerationop import (
        CreateImageGenerationResponseBody,
        CreateImageGenerationResponseBodyTypedDict,
    )
    from .imageobject import ImageObject, ImageObjectTypedDict
    from .imageurl import Detail, ImageURL, ImageURLTypedDict
    from .jsonschema import JSONSchema, JSONSchemaTypedDict
    from .message import Content, ContentTypedDict, Message, MessageTypedDict, Role
    from .messagecontent import MessageContent, MessageContentTypedDict, Type
    from .model import Model, ModelTypedDict
    from .modellistresponse import ModelListResponse, ModelListResponseTypedDict
    from .moderationcategories import (
        ModerationCategories,
        ModerationCategoriesTypedDict,
    )
    from .moderationcategoryscores import (
        ModerationCategoryScores,
        ModerationCategoryScoresTypedDict,
    )
    from .moderationrequest import (
        ModerationRequest,
        ModerationRequestInput,
        ModerationRequestInputTypedDict,
        ModerationRequestTypedDict,
    )
    from .moderationresponse import ModerationResponse, ModerationResponseTypedDict
    from .moderationresult import ModerationResult, ModerationResultTypedDict
    from .responseformat import (
        ResponseFormat,
        ResponseFormatType,
        ResponseFormatTypedDict,
    )
    from .responseobject import (
        Billing,
        BillingTypedDict,
        Error,
        ErrorTypedDict,
        IncompleteDetails,
        IncompleteDetailsTypedDict,
        Payer,
        ResponseObject,
        ResponseObject1,
        ResponseObject2,
        ResponseObject2TypedDict,
        ResponseObjectEffort,
        ResponseObjectFormat,
        ResponseObjectFormatTypedDict,
        ResponseObjectReasoning,
        ResponseObjectReasoningTypedDict,
        ResponseObjectServiceTier,
        ResponseObjectText,
        ResponseObjectTextTypedDict,
        ResponseObjectToolChoice,
        ResponseObjectToolChoiceTypedDict,
        ResponseObjectTruncation,
        ResponseObjectType,
        ResponseObjectTypedDict,
        ResponseObjectVerbosity,
        Status,
    )
    from .responseoutputitem import (
        ResponseOutputItem,
        ResponseOutputItemContent,
        ResponseOutputItemContentTypedDict,
        ResponseOutputItemRole,
        ResponseOutputItemSchemasType,
        ResponseOutputItemStatus,
        ResponseOutputItemType,
        ResponseOutputItemTypedDict,
        Summary,
        SummaryTypedDict,
    )
    from .responserequest import (
        Effort,
        Input,
        InputTypedDict,
        Reasoning,
        ReasoningTypedDict,
        ResponseRequest,
        ResponseRequest1,
        ResponseRequestFormat,
        ResponseRequestFormatTypedDict,
        ResponseRequestModalities,
        ResponseRequestStop,
        ResponseRequestStopTypedDict,
        ResponseRequestToolChoice,
        ResponseRequestToolChoiceTypedDict,
        ResponseRequestType,
        ResponseRequestTypedDict,
        Text,
        TextTypedDict,
        Truncation,
        Verbosity,
    )
    from .responsestreamevent import (
        Delta,
        DeltaTypedDict,
        ResponseStreamEvent,
        ResponseStreamEvent2,
        ResponseStreamEvent2TypedDict,
        ResponseStreamEventTypedDict,
    )
    from .responsetool import (
        Parameters,
        ParametersTypedDict,
        ResponseTool,
        ResponseToolSchemasType,
        ResponseToolType,
        ResponseToolTypedDict,
    )
    from .responseusage import (
        InputTokensDetails,
        InputTokensDetailsTypedDict,
        OutputTokensDetails,
        OutputTokensDetailsTypedDict,
        ResponseUsage,
        ResponseUsageTypedDict,
    )
    from .retrievemodelop import RetrieveModelRequest, RetrieveModelRequestTypedDict
    from .searchrequest import (
        Engine,
        Extra,
        ExtraTypedDict,
        SearchRequest,
        SearchRequestTypedDict,
    )
    from .searchresponse import (
        SearchResponse,
        SearchResponseEngine,
        SearchResponseTypedDict,
    )
    from .security import Security, SecurityTypedDict
    from .streamoptions import StreamOptions, StreamOptionsTypedDict
    from .tool import Function, FunctionTypedDict, Tool, ToolType, ToolTypedDict
    from .toolcall import ToolCall, ToolCallTypedDict
    from .toolchoice import (
        ToolChoice,
        ToolChoiceFunction,
        ToolChoiceFunctionTypedDict,
        ToolChoiceTypedDict,
    )
    from .usage import (
        CompletionTokensDetails,
        CompletionTokensDetailsTypedDict,
        PromptTokensDetails,
        PromptTokensDetailsTypedDict,
        Usage,
        UsageTypedDict,
    )

__all__ = [
    "AnthropicContent",
    "AnthropicContentBlockDelta",
    "AnthropicContentBlockDeltaDelta",
    "AnthropicContentBlockDeltaDeltaTypedDict",
    "AnthropicContentBlockDeltaType",
    "AnthropicContentBlockDeltaTypedDict",
    "AnthropicContentBlockStart",
    "AnthropicContentBlockStartInput",
    "AnthropicContentBlockStartInputTypedDict",
    "AnthropicContentBlockStartType",
    "AnthropicContentBlockStartTypedDict",
    "AnthropicContentBlockStop",
    "AnthropicContentBlockStopContentBlock",
    "AnthropicContentBlockStopContentBlockTypedDict",
    "AnthropicContentBlockStopTypedDict",
    "AnthropicContentTypedDict",
    "AnthropicDocumentContent",
    "AnthropicDocumentContentTypedDict",
    "AnthropicDocumentSource",
    "AnthropicDocumentSourceBase64",
    "AnthropicDocumentSourceBase64MediaType",
    "AnthropicDocumentSourceBase64TypedDict",
    "AnthropicDocumentSourceTypedDict",
    "AnthropicDocumentSourceURL",
    "AnthropicDocumentSourceURLTypedDict",
    "AnthropicImageContent",
    "AnthropicImageContentTypedDict",
    "AnthropicImageSource",
    "AnthropicImageSourceBase64",
    "AnthropicImageSourceBase64TypedDict",
    "AnthropicImageSourceTypedDict",
    "AnthropicImageSourceURL",
    "AnthropicImageSourceURLTypedDict",
    "AnthropicInputSchema",
    "AnthropicInputSchemaTypedDict",
    "AnthropicMessageDelta",
    "AnthropicMessageDeltaDelta",
    "AnthropicMessageDeltaDeltaTypedDict",
    "AnthropicMessageDeltaTypedDict",
    "AnthropicMessageDeltaUsage",
    "AnthropicMessageDeltaUsageTypedDict",
    "AnthropicMessageMessage",
    "AnthropicMessageMessageContent",
    "AnthropicMessageMessageContentTypedDict",
    "AnthropicMessageMessageRole",
    "AnthropicMessageMessageTypedDict",
    "AnthropicMessageRequest",
    "AnthropicMessageRequest1",
    "AnthropicMessageRequest2",
    "AnthropicMessageRequest2TypedDict",
    "AnthropicMessageRequestServiceTier",
    "AnthropicMessageRequestToolChoice",
    "AnthropicMessageRequestToolChoiceTypedDict",
    "AnthropicMessageRequestType",
    "AnthropicMessageRequestTypedDict",
    "AnthropicMessageResponse",
    "AnthropicMessageResponseTypedDict",
    "AnthropicMessageResponseUsage",
    "AnthropicMessageResponseUsageTypedDict",
    "AnthropicMessageStart",
    "AnthropicMessageStartContent",
    "AnthropicMessageStartContentTypedDict",
    "AnthropicMessageStartMessage",
    "AnthropicMessageStartMessageTypedDict",
    "AnthropicMessageStartTypedDict",
    "AnthropicMessageStartUsage",
    "AnthropicMessageStartUsageTypedDict",
    "AnthropicPing",
    "AnthropicPingTypedDict",
    "AnthropicStreamEvent",
    "AnthropicStreamEventTypedDict",
    "AnthropicStreamMessageStop",
    "AnthropicStreamMessageStopTypedDict",
    "AnthropicTextContent",
    "AnthropicTextContentTypedDict",
    "AnthropicThinkingContent",
    "AnthropicThinkingContentTypedDict",
    "AnthropicTool",
    "AnthropicToolContent",
    "AnthropicToolContentTypedDict",
    "AnthropicToolResultContent",
    "AnthropicToolResultContentTypedDict",
    "AnthropicToolTypedDict",
    "AnthropicToolUseContent",
    "AnthropicToolUseContentTypedDict",
    "Audio",
    "AudioSpeechRequest",
    "AudioSpeechRequestResponseFormat",
    "AudioSpeechRequestTypedDict",
    "AudioSpeechRequestVoice",
    "AudioTranscriptionRequest",
    "AudioTranscriptionRequestResponseFormat",
    "AudioTranscriptionRequestTypedDict",
    "AudioTranscriptionResponse",
    "AudioTranscriptionResponseTypedDict",
    "AudioTranslationRequest",
    "AudioTranslationRequestFile",
    "AudioTranslationRequestFileTypedDict",
    "AudioTranslationRequestResponseFormat",
    "AudioTranslationRequestTypedDict",
    "AudioTranslationResponse",
    "AudioTranslationResponseTypedDict",
    "AudioTypedDict",
    "Billing",
    "BillingTypedDict",
    "ChatCompletionChoice",
    "ChatCompletionChoiceTypedDict",
    "ChatCompletionRequest",
    "ChatCompletionRequestFunction",
    "ChatCompletionRequestFunctionTypedDict",
    "ChatCompletionRequestToolChoice",
    "ChatCompletionRequestToolChoiceTypedDict",
    "ChatCompletionRequestTypedDict",
    "ChatCompletionResponse",
    "ChatCompletionResponseTypedDict",
    "ChatCompletionStreamChoice",
    "ChatCompletionStreamChoiceFinishReason",
    "ChatCompletionStreamChoiceLogprobs",
    "ChatCompletionStreamChoiceLogprobsTypedDict",
    "ChatCompletionStreamChoiceTypedDict",
    "ChatCompletionStreamDelta",
    "ChatCompletionStreamDeltaRole",
    "ChatCompletionStreamDeltaTypedDict",
    "ChatCompletionStreamEvent",
    "ChatCompletionStreamEventTypedDict",
    "CompletionChoice",
    "CompletionChoiceFinishReason",
    "CompletionChoiceLogprobs",
    "CompletionChoiceLogprobsTypedDict",
    "CompletionChoiceTypedDict",
    "CompletionRequest",
    "CompletionRequestStop",
    "CompletionRequestStopTypedDict",
    "CompletionRequestTypedDict",
    "CompletionResponse",
    "CompletionResponseTypedDict",
    "CompletionStreamChoice",
    "CompletionStreamChoiceFinishReason",
    "CompletionStreamChoiceLogprobs",
    "CompletionStreamChoiceLogprobsTypedDict",
    "CompletionStreamChoiceTypedDict",
    "CompletionStreamEvent",
    "CompletionStreamEventTypedDict",
    "CompletionTokensDetails",
    "CompletionTokensDetailsTypedDict",
    "Content",
    "ContentBlock",
    "ContentBlockTypedDict",
    "ContentTypedDict",
    "CreateAudioSpeechResponse",
    "CreateAudioSpeechResponseTypedDict",
    "CreateAudioTranscriptionResponse",
    "CreateAudioTranscriptionResponseTypedDict",
    "CreateAudioTranslationResponse",
    "CreateAudioTranslationResponseTypedDict",
    "CreateChatCompletionResponse",
    "CreateChatCompletionResponseBody",
    "CreateChatCompletionResponseBodyTypedDict",
    "CreateChatCompletionResponseTypedDict",
    "CreateCompletionResponse",
    "CreateCompletionResponseBody",
    "CreateCompletionResponseBodyTypedDict",
    "CreateCompletionResponseTypedDict",
    "CreateEngineEmbeddingRequest",
    "CreateEngineEmbeddingRequestTypedDict",
    "CreateMessageResponse",
    "CreateMessageResponseBody",
    "CreateMessageResponseBodyTypedDict",
    "CreateMessageResponseTypedDict",
    "CreateProxyRequestRequest",
    "CreateProxyRequestRequestTypedDict",
    "CreateResponseResponse",
    "CreateResponseResponseBody",
    "CreateResponseResponseBodyTypedDict",
    "CreateResponseResponseTypedDict",
    "Delta",
    "DeltaTypedDict",
    "Detail",
    "EditChoice",
    "EditChoiceTypedDict",
    "EditRequest",
    "EditRequestTypedDict",
    "EditResponse",
    "EditResponseTypedDict",
    "Effort",
    "EmbeddingObject",
    "EmbeddingObjectTypedDict",
    "EmbeddingRequest",
    "EmbeddingRequestInput",
    "EmbeddingRequestInputTypedDict",
    "EmbeddingRequestTypedDict",
    "EmbeddingResponse",
    "EmbeddingResponseTypedDict",
    "EncodingFormat",
    "Engine",
    "EngineEmbeddingRequest",
    "EngineEmbeddingRequestEncodingFormat",
    "EngineEmbeddingRequestInput",
    "EngineEmbeddingRequestInputTypedDict",
    "EngineEmbeddingRequestTypedDict",
    "Error",
    "ErrorTypedDict",
    "Extra",
    "ExtraTypedDict",
    "File",
    "FileTypedDict",
    "FinishReason",
    "Format",
    "Function",
    "FunctionCall",
    "FunctionCallTypedDict",
    "FunctionTypedDict",
    "EditImageResponseBody",
    "EditImageResponseBodyTypedDict",
    "ImageEditEventData",
    "ImageEditEventDataTypedDict",
    "ImageEditBackground",
    "ImageEditInputFidelity",
    "ImageEditModeration",
    "ImageEditOutputFormat",
    "ImageEditQuality",
    "ImageEditRequest",
    "ImageEditRequestTypedDict",
    "ImageEditResponseFormat",
    "ImageEditSize",
    "ImageEditStreamEvent",
    "ImageEditStreamEventTypedDict",
    "ImageFile",
    "ImageFileTypedDict",
    "ImageGenerationBackground",
    "ImageGenerationModeration",
    "ImageGenerationOutputFormat",
    "ImageGenerationRequest",
    "ImageGenerationRequestResponseFormat",
    "ImageGenerationRequestTypedDict",
    "ImageGenerationResponse",
    "ImageGenerationResponseTypedDict",
    "ImageGenerationStreamData",
    "ImageGenerationStreamDataTypedDict",
    "ImageGenerationStreamEvent",
    "ImageGenerationStreamEventTypedDict",
    "CreateImageGenerationResponseBody",
    "CreateImageGenerationResponseBodyTypedDict",
    "ImageUsage",
    "ImageUsageTypedDict",
    "ImageObject",
    "ImageObjectTypedDict",
    "ImageURL",
    "ImageURLTypedDict",
    "IncompleteDetails",
    "IncompleteDetailsTypedDict",
    "Input",
    "InputTokensDetails",
    "InputTokensDetailsTypedDict",
    "InputTypedDict",
    "JSONSchema",
    "JSONSchemaTypedDict",
    "Logprobs",
    "LogprobsTypedDict",
    "MediaType",
    "Message",
    "MessageContent",
    "MessageContentTypedDict",
    "MessageTypedDict",
    "Modalities",
    "Model",
    "ModelListResponse",
    "ModelListResponseTypedDict",
    "ModelTypedDict",
    "ModerationCategories",
    "ModerationCategoriesTypedDict",
    "ModerationCategoryScores",
    "ModerationCategoryScoresTypedDict",
    "ModerationRequest",
    "ModerationRequestInput",
    "ModerationRequestInputTypedDict",
    "ModerationRequestTypedDict",
    "ModerationResponse",
    "ModerationResponseTypedDict",
    "ModerationResult",
    "ModerationResultTypedDict",
    "One",
    "OutputTokensDetails",
    "OutputTokensDetailsTypedDict",
    "Parameters",
    "ParametersTypedDict",
    "Payer",
    "PromptTokensDetails",
    "PromptTokensDetailsTypedDict",
    "Properties",
    "PropertiesTypedDict",
    "Quality",
    "Reasoning",
    "ReasoningEffort",
    "ReasoningTypedDict",
    "ResponseFormat",
    "ResponseFormatType",
    "ResponseFormatTypedDict",
    "ResponseObject",
    "ResponseObject1",
    "ResponseObject2",
    "ResponseObject2TypedDict",
    "ResponseObjectEffort",
    "ResponseObjectFormat",
    "ResponseObjectFormatTypedDict",
    "ResponseObjectReasoning",
    "ResponseObjectReasoningTypedDict",
    "ResponseObjectServiceTier",
    "ResponseObjectText",
    "ResponseObjectTextTypedDict",
    "ResponseObjectToolChoice",
    "ResponseObjectToolChoiceTypedDict",
    "ResponseObjectTruncation",
    "ResponseObjectType",
    "ResponseObjectTypedDict",
    "ResponseObjectVerbosity",
    "ResponseOutputItem",
    "ResponseOutputItemContent",
    "ResponseOutputItemContentTypedDict",
    "ResponseOutputItemRole",
    "ResponseOutputItemSchemasType",
    "ResponseOutputItemStatus",
    "ResponseOutputItemType",
    "ResponseOutputItemTypedDict",
    "ResponseRequest",
    "ResponseRequest1",
    "ResponseRequestFormat",
    "ResponseRequestFormatTypedDict",
    "ResponseRequestModalities",
    "ResponseRequestStop",
    "ResponseRequestStopTypedDict",
    "ResponseRequestToolChoice",
    "ResponseRequestToolChoiceTypedDict",
    "ResponseRequestType",
    "ResponseRequestTypedDict",
    "ResponseStreamEvent",
    "ResponseStreamEvent2",
    "ResponseStreamEvent2TypedDict",
    "ResponseStreamEventTypedDict",
    "ResponseTool",
    "ResponseToolSchemasType",
    "ResponseToolType",
    "ResponseToolTypedDict",
    "ResponseUsage",
    "ResponseUsageTypedDict",
    "RetrieveModelRequest",
    "RetrieveModelRequestTypedDict",
    "Role",
    "SearchRequest",
    "SearchRequestTypedDict",
    "SearchResponse",
    "SearchResponseEngine",
    "SearchResponseTypedDict",
    "Security",
    "SecurityTypedDict",
    "Segments",
    "SegmentsTypedDict",
    "ServiceTier",
    "Size",
    "Status",
    "Stop",
    "StopReason",
    "StopTypedDict",
    "StreamOptions",
    "StreamOptionsTypedDict",
    "Style",
    "Summary",
    "SummaryTypedDict",
    "Text",
    "TextTypedDict",
    "Thinking",
    "ThinkingTypedDict",
    "TimestampGranularities",
    "Tool",
    "ToolCall",
    "ToolCallTypedDict",
    "ToolChoice",
    "ToolChoiceFunction",
    "ToolChoiceFunctionTypedDict",
    "ToolChoiceTypedDict",
    "ToolType",
    "ToolTypedDict",
    "Truncation",
    "Two",
    "TwoTypedDict",
    "Type",
    "Usage",
    "UsageTypedDict",
    "Verbosity",
    "Voice",
    "Words",
    "WordsTypedDict",
]

_dynamic_imports: dict[str, str] = {
    "AnthropicContent": ".anthropiccontent",
    "AnthropicContentTypedDict": ".anthropiccontent",
    "AnthropicContentBlockDelta": ".anthropiccontentblockdelta",
    "AnthropicContentBlockDeltaDelta": ".anthropiccontentblockdelta",
    "AnthropicContentBlockDeltaDeltaTypedDict": ".anthropiccontentblockdelta",
    "AnthropicContentBlockDeltaType": ".anthropiccontentblockdelta",
    "AnthropicContentBlockDeltaTypedDict": ".anthropiccontentblockdelta",
    "AnthropicContentBlockStart": ".anthropiccontentblockstart",
    "AnthropicContentBlockStartInput": ".anthropiccontentblockstart",
    "AnthropicContentBlockStartInputTypedDict": ".anthropiccontentblockstart",
    "AnthropicContentBlockStartType": ".anthropiccontentblockstart",
    "AnthropicContentBlockStartTypedDict": ".anthropiccontentblockstart",
    "ContentBlock": ".anthropiccontentblockstart",
    "ContentBlockTypedDict": ".anthropiccontentblockstart",
    "AnthropicContentBlockStop": ".anthropiccontentblockstop",
    "AnthropicContentBlockStopContentBlock": ".anthropiccontentblockstop",
    "AnthropicContentBlockStopContentBlockTypedDict": ".anthropiccontentblockstop",
    "AnthropicContentBlockStopTypedDict": ".anthropiccontentblockstop",
    "AnthropicDocumentContent": ".anthropicdocumentcontent",
    "AnthropicDocumentContentTypedDict": ".anthropicdocumentcontent",
    "AnthropicDocumentSource": ".anthropicdocumentsource",
    "AnthropicDocumentSourceTypedDict": ".anthropicdocumentsource",
    "AnthropicDocumentSourceBase64": ".anthropicdocumentsourcebase64",
    "AnthropicDocumentSourceBase64MediaType": ".anthropicdocumentsourcebase64",
    "AnthropicDocumentSourceBase64TypedDict": ".anthropicdocumentsourcebase64",
    "AnthropicDocumentSourceURL": ".anthropicdocumentsourceurl",
    "AnthropicDocumentSourceURLTypedDict": ".anthropicdocumentsourceurl",
    "AnthropicImageContent": ".anthropicimagecontent",
    "AnthropicImageContentTypedDict": ".anthropicimagecontent",
    "AnthropicImageSource": ".anthropicimagesource",
    "AnthropicImageSourceTypedDict": ".anthropicimagesource",
    "AnthropicImageSourceBase64": ".anthropicimagesourcebase64",
    "AnthropicImageSourceBase64TypedDict": ".anthropicimagesourcebase64",
    "MediaType": ".anthropicimagesourcebase64",
    "AnthropicImageSourceURL": ".anthropicimagesourceurl",
    "AnthropicImageSourceURLTypedDict": ".anthropicimagesourceurl",
    "AnthropicInputSchema": ".anthropicinputschema",
    "AnthropicInputSchemaTypedDict": ".anthropicinputschema",
    "Properties": ".anthropicinputschema",
    "PropertiesTypedDict": ".anthropicinputschema",
    "AnthropicMessageDelta": ".anthropicmessagedelta",
    "AnthropicMessageDeltaDelta": ".anthropicmessagedelta",
    "AnthropicMessageDeltaDeltaTypedDict": ".anthropicmessagedelta",
    "AnthropicMessageDeltaTypedDict": ".anthropicmessagedelta",
    "AnthropicMessageDeltaUsage": ".anthropicmessagedelta",
    "AnthropicMessageDeltaUsageTypedDict": ".anthropicmessagedelta",
    "AnthropicMessageMessage": ".anthropicmessagemessage",
    "AnthropicMessageMessageContent": ".anthropicmessagemessage",
    "AnthropicMessageMessageContentTypedDict": ".anthropicmessagemessage",
    "AnthropicMessageMessageRole": ".anthropicmessagemessage",
    "AnthropicMessageMessageTypedDict": ".anthropicmessagemessage",
    "AnthropicMessageRequest": ".anthropicmessagerequest",
    "AnthropicMessageRequest1": ".anthropicmessagerequest",
    "AnthropicMessageRequest2": ".anthropicmessagerequest",
    "AnthropicMessageRequest2TypedDict": ".anthropicmessagerequest",
    "AnthropicMessageRequestServiceTier": ".anthropicmessagerequest",
    "AnthropicMessageRequestToolChoice": ".anthropicmessagerequest",
    "AnthropicMessageRequestToolChoiceTypedDict": ".anthropicmessagerequest",
    "AnthropicMessageRequestType": ".anthropicmessagerequest",
    "AnthropicMessageRequestTypedDict": ".anthropicmessagerequest",
    "Thinking": ".anthropicmessagerequest",
    "ThinkingTypedDict": ".anthropicmessagerequest",
    "AnthropicMessageResponse": ".anthropicmessageresponse",
    "AnthropicMessageResponseTypedDict": ".anthropicmessageresponse",
    "AnthropicMessageResponseUsage": ".anthropicmessageresponse",
    "AnthropicMessageResponseUsageTypedDict": ".anthropicmessageresponse",
    "StopReason": ".anthropicmessageresponse",
    "AnthropicMessageStart": ".anthropicmessagestart",
    "AnthropicMessageStartContent": ".anthropicmessagestart",
    "AnthropicMessageStartContentTypedDict": ".anthropicmessagestart",
    "AnthropicMessageStartMessage": ".anthropicmessagestart",
    "AnthropicMessageStartMessageTypedDict": ".anthropicmessagestart",
    "AnthropicMessageStartTypedDict": ".anthropicmessagestart",
    "AnthropicMessageStartUsage": ".anthropicmessagestart",
    "AnthropicMessageStartUsageTypedDict": ".anthropicmessagestart",
    "AnthropicPing": ".anthropicping",
    "AnthropicPingTypedDict": ".anthropicping",
    "AnthropicStreamEvent": ".anthropicstreamevent",
    "AnthropicStreamEventTypedDict": ".anthropicstreamevent",
    "AnthropicStreamMessageStop": ".anthropicstreammessagestop",
    "AnthropicStreamMessageStopTypedDict": ".anthropicstreammessagestop",
    "AnthropicTextContent": ".anthropictextcontent",
    "AnthropicTextContentTypedDict": ".anthropictextcontent",
    "AnthropicThinkingContent": ".anthropicthinkingcontent",
    "AnthropicThinkingContentTypedDict": ".anthropicthinkingcontent",
    "AnthropicTool": ".anthropictool",
    "AnthropicToolTypedDict": ".anthropictool",
    "AnthropicToolContent": ".anthropictoolcontent",
    "AnthropicToolContentTypedDict": ".anthropictoolcontent",
    "AnthropicToolResultContent": ".anthropictoolresultcontent",
    "AnthropicToolResultContentTypedDict": ".anthropictoolresultcontent",
    "AnthropicToolUseContent": ".anthropictoolusecontent",
    "AnthropicToolUseContentTypedDict": ".anthropictoolusecontent",
    "Audio": ".audio",
    "AudioTypedDict": ".audio",
    "Format": ".audio",
    "Voice": ".audio",
    "AudioSpeechRequest": ".audiospeechrequest",
    "AudioSpeechRequestResponseFormat": ".audiospeechrequest",
    "AudioSpeechRequestTypedDict": ".audiospeechrequest",
    "AudioSpeechRequestVoice": ".audiospeechrequest",
    "AudioTranscriptionRequest": ".audiotranscriptionrequest",
    "AudioTranscriptionRequestResponseFormat": ".audiotranscriptionrequest",
    "AudioTranscriptionRequestTypedDict": ".audiotranscriptionrequest",
    "File": ".audiotranscriptionrequest",
    "FileTypedDict": ".audiotranscriptionrequest",
    "TimestampGranularities": ".audiotranscriptionrequest",
    "AudioTranscriptionResponse": ".audiotranscriptionresponse",
    "AudioTranscriptionResponseTypedDict": ".audiotranscriptionresponse",
    "Segments": ".audiotranscriptionresponse",
    "SegmentsTypedDict": ".audiotranscriptionresponse",
    "Words": ".audiotranscriptionresponse",
    "WordsTypedDict": ".audiotranscriptionresponse",
    "AudioTranslationRequest": ".audiotranslationrequest",
    "AudioTranslationRequestFile": ".audiotranslationrequest",
    "AudioTranslationRequestFileTypedDict": ".audiotranslationrequest",
    "AudioTranslationRequestResponseFormat": ".audiotranslationrequest",
    "AudioTranslationRequestTypedDict": ".audiotranslationrequest",
    "AudioTranslationResponse": ".audiotranslationresponse",
    "AudioTranslationResponseTypedDict": ".audiotranslationresponse",
    "ChatCompletionChoice": ".chatcompletionchoice",
    "ChatCompletionChoiceTypedDict": ".chatcompletionchoice",
    "FinishReason": ".chatcompletionchoice",
    "Logprobs": ".chatcompletionchoice",
    "LogprobsTypedDict": ".chatcompletionchoice",
    "ChatCompletionRequest": ".chatcompletionrequest",
    "ChatCompletionRequestFunction": ".chatcompletionrequest",
    "ChatCompletionRequestFunctionTypedDict": ".chatcompletionrequest",
    "ChatCompletionRequestToolChoice": ".chatcompletionrequest",
    "ChatCompletionRequestToolChoiceTypedDict": ".chatcompletionrequest",
    "ChatCompletionRequestTypedDict": ".chatcompletionrequest",
    "Modalities": ".chatcompletionrequest",
    "One": ".chatcompletionrequest",
    "ReasoningEffort": ".chatcompletionrequest",
    "ServiceTier": ".chatcompletionrequest",
    "Stop": ".chatcompletionrequest",
    "StopTypedDict": ".chatcompletionrequest",
    "Two": ".chatcompletionrequest",
    "TwoTypedDict": ".chatcompletionrequest",
    "ChatCompletionResponse": ".chatcompletionresponse",
    "ChatCompletionResponseTypedDict": ".chatcompletionresponse",
    "ChatCompletionStreamChoice": ".chatcompletionstreamchoice",
    "ChatCompletionStreamChoiceFinishReason": ".chatcompletionstreamchoice",
    "ChatCompletionStreamChoiceLogprobs": ".chatcompletionstreamchoice",
    "ChatCompletionStreamChoiceLogprobsTypedDict": ".chatcompletionstreamchoice",
    "ChatCompletionStreamChoiceTypedDict": ".chatcompletionstreamchoice",
    "ChatCompletionStreamDelta": ".chatcompletionstreamdelta",
    "ChatCompletionStreamDeltaRole": ".chatcompletionstreamdelta",
    "ChatCompletionStreamDeltaTypedDict": ".chatcompletionstreamdelta",
    "ChatCompletionStreamEvent": ".chatcompletionstreamevent",
    "ChatCompletionStreamEventTypedDict": ".chatcompletionstreamevent",
    "CompletionChoice": ".completionchoice",
    "CompletionChoiceFinishReason": ".completionchoice",
    "CompletionChoiceLogprobs": ".completionchoice",
    "CompletionChoiceLogprobsTypedDict": ".completionchoice",
    "CompletionChoiceTypedDict": ".completionchoice",
    "CompletionRequest": ".completionrequest",
    "CompletionRequestStop": ".completionrequest",
    "CompletionRequestStopTypedDict": ".completionrequest",
    "CompletionRequestTypedDict": ".completionrequest",
    "CompletionResponse": ".completionresponse",
    "CompletionResponseTypedDict": ".completionresponse",
    "CompletionStreamChoice": ".completionstreamchoice",
    "CompletionStreamChoiceFinishReason": ".completionstreamchoice",
    "CompletionStreamChoiceLogprobs": ".completionstreamchoice",
    "CompletionStreamChoiceLogprobsTypedDict": ".completionstreamchoice",
    "CompletionStreamChoiceTypedDict": ".completionstreamchoice",
    "CompletionStreamEvent": ".completionstreamevent",
    "CompletionStreamEventTypedDict": ".completionstreamevent",
    "CreateAudioSpeechResponse": ".createaudiospeechop",
    "CreateAudioSpeechResponseTypedDict": ".createaudiospeechop",
    "CreateAudioTranscriptionResponse": ".createaudiotranscriptionop",
    "CreateAudioTranscriptionResponseTypedDict": ".createaudiotranscriptionop",
    "CreateAudioTranslationResponse": ".createaudiotranslationop",
    "CreateAudioTranslationResponseTypedDict": ".createaudiotranslationop",
    "CreateChatCompletionResponse": ".createchatcompletionop",
    "CreateChatCompletionResponseBody": ".createchatcompletionop",
    "CreateChatCompletionResponseBodyTypedDict": ".createchatcompletionop",
    "CreateChatCompletionResponseTypedDict": ".createchatcompletionop",
    "CreateCompletionResponse": ".createcompletionop",
    "CreateCompletionResponseBody": ".createcompletionop",
    "CreateCompletionResponseBodyTypedDict": ".createcompletionop",
    "CreateCompletionResponseTypedDict": ".createcompletionop",
    "CreateEngineEmbeddingRequest": ".createengineembeddingop",
    "CreateEngineEmbeddingRequestTypedDict": ".createengineembeddingop",
    "CreateMessageResponse": ".createmessageop",
    "CreateMessageResponseBody": ".createmessageop",
    "CreateMessageResponseBodyTypedDict": ".createmessageop",
    "CreateMessageResponseTypedDict": ".createmessageop",
    "CreateProxyRequestRequest": ".createproxyrequestop",
    "CreateProxyRequestRequestTypedDict": ".createproxyrequestop",
    "CreateResponseResponse": ".createresponseop",
    "CreateResponseResponseBody": ".createresponseop",
    "CreateResponseResponseBodyTypedDict": ".createresponseop",
    "CreateResponseResponseTypedDict": ".createresponseop",
    "EditChoice": ".editchoice",
    "EditChoiceTypedDict": ".editchoice",
    "EditRequest": ".editrequest",
    "EditRequestTypedDict": ".editrequest",
    "EditResponse": ".editresponse",
    "EditResponseTypedDict": ".editresponse",
    "EmbeddingObject": ".embeddingobject",
    "EmbeddingObjectTypedDict": ".embeddingobject",
    "EmbeddingRequest": ".embeddingrequest",
    "EmbeddingRequestInput": ".embeddingrequest",
    "EmbeddingRequestInputTypedDict": ".embeddingrequest",
    "EmbeddingRequestTypedDict": ".embeddingrequest",
    "EncodingFormat": ".embeddingrequest",
    "EmbeddingResponse": ".embeddingresponse",
    "EmbeddingResponseTypedDict": ".embeddingresponse",
    "EngineEmbeddingRequest": ".engineembeddingrequest",
    "EngineEmbeddingRequestEncodingFormat": ".engineembeddingrequest",
    "EngineEmbeddingRequestInput": ".engineembeddingrequest",
    "EngineEmbeddingRequestInputTypedDict": ".engineembeddingrequest",
    "EngineEmbeddingRequestTypedDict": ".engineembeddingrequest",
    "FunctionCall": ".functioncall",
    "FunctionCallTypedDict": ".functioncall",
    "EditImageResponseBody": ".editimageop",
    "EditImageResponseBodyTypedDict": ".editimageop",
    "ImageEditEventData": ".editimageop",
    "ImageEditEventDataTypedDict": ".editimageop",
    "ImageEditBackground": ".imageeditrequest",
    "ImageEditInputFidelity": ".imageeditrequest",
    "ImageEditModeration": ".imageeditrequest",
    "ImageEditOutputFormat": ".imageeditrequest",
    "ImageEditQuality": ".imageeditrequest",
    "ImageEditRequest": ".imageeditrequest",
    "ImageEditRequestTypedDict": ".imageeditrequest",
    "ImageEditResponseFormat": ".imageeditrequest",
    "ImageEditSize": ".imageeditrequest",
    "ImageEditStreamEvent": ".imageeditstreamevent",
    "ImageEditStreamEventTypedDict": ".imageeditstreamevent",
    "ImageFile": ".imageeditrequest",
    "ImageFileTypedDict": ".imageeditrequest",
    "ImageGenerationBackground": ".imagegenerationrequest",
    "ImageGenerationModeration": ".imagegenerationrequest",
    "ImageGenerationOutputFormat": ".imagegenerationrequest",
    "ImageGenerationRequest": ".imagegenerationrequest",
    "ImageGenerationRequestResponseFormat": ".imagegenerationrequest",
    "ImageGenerationRequestTypedDict": ".imagegenerationrequest",
    "Quality": ".imagegenerationrequest",
    "Size": ".imagegenerationrequest",
    "Style": ".imagegenerationrequest",
    "ImageGenerationResponse": ".imagegenerationresponse",
    "ImageGenerationResponseTypedDict": ".imagegenerationresponse",
    "ImageGenerationStreamData": ".imagegenerationstreamevent",
    "ImageGenerationStreamDataTypedDict": ".imagegenerationstreamevent",
    "ImageGenerationStreamEvent": ".imagegenerationstreamevent",
    "ImageGenerationStreamEventTypedDict": ".imagegenerationstreamevent",
    "CreateImageGenerationResponseBody": ".createimagegenerationop",
    "CreateImageGenerationResponseBodyTypedDict": ".createimagegenerationop",
    "ImageUsage": ".imagegenerationresponse",
    "ImageUsageTypedDict": ".imagegenerationresponse",
    "ImageObject": ".imageobject",
    "ImageObjectTypedDict": ".imageobject",
    "Detail": ".imageurl",
    "ImageURL": ".imageurl",
    "ImageURLTypedDict": ".imageurl",
    "JSONSchema": ".jsonschema",
    "JSONSchemaTypedDict": ".jsonschema",
    "Content": ".message",
    "ContentTypedDict": ".message",
    "Message": ".message",
    "MessageTypedDict": ".message",
    "Role": ".message",
    "MessageContent": ".messagecontent",
    "MessageContentTypedDict": ".messagecontent",
    "Type": ".messagecontent",
    "Model": ".model",
    "ModelTypedDict": ".model",
    "ModelListResponse": ".modellistresponse",
    "ModelListResponseTypedDict": ".modellistresponse",
    "ModerationCategories": ".moderationcategories",
    "ModerationCategoriesTypedDict": ".moderationcategories",
    "ModerationCategoryScores": ".moderationcategoryscores",
    "ModerationCategoryScoresTypedDict": ".moderationcategoryscores",
    "ModerationRequest": ".moderationrequest",
    "ModerationRequestInput": ".moderationrequest",
    "ModerationRequestInputTypedDict": ".moderationrequest",
    "ModerationRequestTypedDict": ".moderationrequest",
    "ModerationResponse": ".moderationresponse",
    "ModerationResponseTypedDict": ".moderationresponse",
    "ModerationResult": ".moderationresult",
    "ModerationResultTypedDict": ".moderationresult",
    "ResponseFormat": ".responseformat",
    "ResponseFormatType": ".responseformat",
    "ResponseFormatTypedDict": ".responseformat",
    "Billing": ".responseobject",
    "BillingTypedDict": ".responseobject",
    "Error": ".responseobject",
    "ErrorTypedDict": ".responseobject",
    "IncompleteDetails": ".responseobject",
    "IncompleteDetailsTypedDict": ".responseobject",
    "Payer": ".responseobject",
    "ResponseObject": ".responseobject",
    "ResponseObject1": ".responseobject",
    "ResponseObject2": ".responseobject",
    "ResponseObject2TypedDict": ".responseobject",
    "ResponseObjectEffort": ".responseobject",
    "ResponseObjectFormat": ".responseobject",
    "ResponseObjectFormatTypedDict": ".responseobject",
    "ResponseObjectReasoning": ".responseobject",
    "ResponseObjectReasoningTypedDict": ".responseobject",
    "ResponseObjectServiceTier": ".responseobject",
    "ResponseObjectText": ".responseobject",
    "ResponseObjectTextTypedDict": ".responseobject",
    "ResponseObjectToolChoice": ".responseobject",
    "ResponseObjectToolChoiceTypedDict": ".responseobject",
    "ResponseObjectTruncation": ".responseobject",
    "ResponseObjectType": ".responseobject",
    "ResponseObjectTypedDict": ".responseobject",
    "ResponseObjectVerbosity": ".responseobject",
    "Status": ".responseobject",
    "ResponseOutputItem": ".responseoutputitem",
    "ResponseOutputItemContent": ".responseoutputitem",
    "ResponseOutputItemContentTypedDict": ".responseoutputitem",
    "ResponseOutputItemRole": ".responseoutputitem",
    "ResponseOutputItemSchemasType": ".responseoutputitem",
    "ResponseOutputItemStatus": ".responseoutputitem",
    "ResponseOutputItemType": ".responseoutputitem",
    "ResponseOutputItemTypedDict": ".responseoutputitem",
    "Summary": ".responseoutputitem",
    "SummaryTypedDict": ".responseoutputitem",
    "Effort": ".responserequest",
    "Input": ".responserequest",
    "InputTypedDict": ".responserequest",
    "Reasoning": ".responserequest",
    "ReasoningTypedDict": ".responserequest",
    "ResponseRequest": ".responserequest",
    "ResponseRequest1": ".responserequest",
    "ResponseRequestFormat": ".responserequest",
    "ResponseRequestFormatTypedDict": ".responserequest",
    "ResponseRequestModalities": ".responserequest",
    "ResponseRequestStop": ".responserequest",
    "ResponseRequestStopTypedDict": ".responserequest",
    "ResponseRequestToolChoice": ".responserequest",
    "ResponseRequestToolChoiceTypedDict": ".responserequest",
    "ResponseRequestType": ".responserequest",
    "ResponseRequestTypedDict": ".responserequest",
    "Text": ".responserequest",
    "TextTypedDict": ".responserequest",
    "Truncation": ".responserequest",
    "Verbosity": ".responserequest",
    "Delta": ".responsestreamevent",
    "DeltaTypedDict": ".responsestreamevent",
    "ResponseStreamEvent": ".responsestreamevent",
    "ResponseStreamEvent2": ".responsestreamevent",
    "ResponseStreamEvent2TypedDict": ".responsestreamevent",
    "ResponseStreamEventTypedDict": ".responsestreamevent",
    "Parameters": ".responsetool",
    "ParametersTypedDict": ".responsetool",
    "ResponseTool": ".responsetool",
    "ResponseToolSchemasType": ".responsetool",
    "ResponseToolType": ".responsetool",
    "ResponseToolTypedDict": ".responsetool",
    "InputTokensDetails": ".responseusage",
    "InputTokensDetailsTypedDict": ".responseusage",
    "OutputTokensDetails": ".responseusage",
    "OutputTokensDetailsTypedDict": ".responseusage",
    "ResponseUsage": ".responseusage",
    "ResponseUsageTypedDict": ".responseusage",
    "RetrieveModelRequest": ".retrievemodelop",
    "RetrieveModelRequestTypedDict": ".retrievemodelop",
    "Engine": ".searchrequest",
    "Extra": ".searchrequest",
    "ExtraTypedDict": ".searchrequest",
    "SearchRequest": ".searchrequest",
    "SearchRequestTypedDict": ".searchrequest",
    "SearchResponse": ".searchresponse",
    "SearchResponseEngine": ".searchresponse",
    "SearchResponseTypedDict": ".searchresponse",
    "Security": ".security",
    "SecurityTypedDict": ".security",
    "StreamOptions": ".streamoptions",
    "StreamOptionsTypedDict": ".streamoptions",
    "Function": ".tool",
    "FunctionTypedDict": ".tool",
    "Tool": ".tool",
    "ToolType": ".tool",
    "ToolTypedDict": ".tool",
    "ToolCall": ".toolcall",
    "ToolCallTypedDict": ".toolcall",
    "ToolChoice": ".toolchoice",
    "ToolChoiceFunction": ".toolchoice",
    "ToolChoiceFunctionTypedDict": ".toolchoice",
    "ToolChoiceTypedDict": ".toolchoice",
    "CompletionTokensDetails": ".usage",
    "CompletionTokensDetailsTypedDict": ".usage",
    "PromptTokensDetails": ".usage",
    "PromptTokensDetailsTypedDict": ".usage",
    "Usage": ".usage",
    "UsageTypedDict": ".usage",
}


def dynamic_import(modname, retries=3):
    for attempt in range(retries):
        try:
            return import_module(modname, __package__)
        except KeyError:
            # Clear any half-initialized module and retry
            sys.modules.pop(modname, None)
            if attempt == retries - 1:
                break
    raise KeyError(f"Failed to import module '{modname}' after {retries} attempts")


def __getattr__(attr_name: str) -> object:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(
            f"No {attr_name} found in _dynamic_imports for module name -> {__name__} "
        )

    try:
        module = dynamic_import(module_name)
        result = getattr(module, attr_name)
        return result
    except ImportError as e:
        raise ImportError(
            f"Failed to import {attr_name} from {module_name}: {e}"
        ) from e
    except AttributeError as e:
        raise AttributeError(
            f"Failed to get {attr_name} from {module_name}: {e}"
        ) from e


def __dir__():
    lazy_attrs = builtins.list(_dynamic_imports.keys())
    return builtins.sorted(lazy_attrs)



import sys
from pathlib import Path
from typing import Optional

# Python 3.11+ has tomllib built-in
if sys.version_info >= (3, 11):
    import tomllib
else:
    try:
        import tomli as tomllib  # pyright: ignore[reportMissingImports]
    except ImportError:
        tomllib = None  # type: ignore[assignment]

try:
    import tomli_w  # pyright: ignore[reportMissingImports]
except ImportError:
    tomli_w = None  # type: ignore[assignment]

from .base import ToolConfigSetResult, ToolIntegration


class CodexIntegration(ToolIntegration):
    primary_name = "codex"
    aliases = ["codex", "openai-codex"]

    def __init__(self) -> None:
        self._settings_path = Path.home() / ".codex" / "config.toml"
        self._backup_dir = Path.home() / ".r9s" / "backup" / "codex"

    def set_config(
        self,
        *,
        base_url: str,
        api_key: str,
        model: str,
        small_model: str,
        wire_api: str = "responses",
        reasoning_effort: Optional[str] = None,
    ) -> ToolConfigSetResult:
        """Configure Codex to use r9s API.

        Args:
            base_url: API base URL
            api_key: Authentication token
            model: Primary model name
            small_model: Not used for Codex (kept for interface compatibility)
            wire_api: API protocol type (responses/chat/completion)
            reasoning_effort: Optional reasoning effort level (low/medium/high)
        """
        if tomllib is None or tomli_w is None:
            raise SystemExit(
                "Missing TOML dependencies. Install with: pip install tomli tomli-w"
            )

        # Normalize base_url: remove trailing slash and /v1 suffix
        normalized_base_url = base_url.rstrip("/")
        # if normalized_base_url.endswith("/v1"):
        #     normalized_base_url = normalized_base_url[:-3]

        # Create backup if config exists
        backup_path = self._create_backup_if_exists()

        # Read existing config or start fresh
        data = self._read_config()

        # Set top-level model configuration
        data["model"] = model
        data["model_provider"] = "r9s"

        # Set reasoning effort if specified
        if reasoning_effort:
            data["model_reasoning_effort"] = reasoning_effort
        elif "model_reasoning_effort" in data:
            # Remove it if not specified (model might not support it)
            del data["model_reasoning_effort"]

        # Ensure model_providers section exists
        if "model_providers" not in data:
            data["model_providers"] = {}

        # Configure r9s provider
        data["model_providers"]["r9s"] = {
            "name": "R9S",
            "base_url": normalized_base_url,
            "env_key": "R9S_API_KEY",
            "wire_api": wire_api,
        }

        # Note: API key should be set as environment variable
        # Codex reads from system environment, not from config file's [env] section

        # Write config file
        self._settings_path.parent.mkdir(parents=True, exist_ok=True)
        with self._settings_path.open("wb") as fp:
            tomli_w.dump(data, fp)

        return ToolConfigSetResult(
            target_path=self._settings_path, backup_path=backup_path
        )

    def run_executable(self) -> str:
        return "codex"

    def run_env(self, *, api_key: str, base_url: str, model: str) -> dict[str, str]:
        """Return environment variables for running Codex.

        Codex reads API key from the R9S_API_KEY environment variable
        which should already be set by the user (validated by run_cli.py).
        All other configs are passed via --config command line flags.
        """
        return {}

    def run_args(self, *, base_url: str, model: str) -> list[str]:
        """Return additional command line arguments for running Codex.

        Uses --config flags to temporarily create a r9s_temp provider
        that overrides config.toml settings without modifying the file.
        """
        provider_name = "r9s_temp"
        env_key = "R9S_API_KEY"
        wire_api = "responses"

        return [
            "--config", f"model_provider={provider_name}",
            "--config", f"model={model}",
            "--config", f"model_providers.{provider_name}.name={provider_name}",
            "--config", f"model_providers.{provider_name}.base_url={base_url}",
            "--config", f"model_providers.{provider_name}.env_key={env_key}",
            "--config", f"model_providers.{provider_name}.wire_api={wire_api}",
        ]

    def _read_config(self) -> dict:
        """Read existing TOML config or return empty dict."""
        if not self._settings_path.exists():
            return {}
        try:
            with self._settings_path.open("rb") as fp:
                data = tomllib.load(fp)
        except Exception:
            return {}
        return data if isinstance(data, dict) else {}



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
import pydantic
from pydantic.functional_validators import AfterValidator
from r9s.types import BaseModel
from r9s.utils import validate_const
from typing import List, Literal
from typing_extensions import Annotated, TypedDict


class EmbeddingObjectTypedDict(TypedDict):
    embedding: List[float]
    r"""Embedding vector"""
    index: int
    object: Literal["embedding"]


class EmbeddingObject(BaseModel):
    embedding: List[float]
    r"""Embedding vector"""

    index: int

    OBJECT: Annotated[
        Annotated[Literal["embedding"], AfterValidator(validate_const("embedding"))],
        pydantic.Field(alias="object"),
    ] = "embedding"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from r9s import errors, models, utils
from r9s._hooks import HookContext
from r9s.types import OptionalNullable, UNSET
from r9s.utils import eventstreaming
from r9s.utils.unmarshal_json_response import unmarshal_json_response
from typing import Any, Literal, Mapping, Optional, Union, overload


class Images(BaseSDK):
    @overload
    def create(
        self,
        *,
        prompt: str,
        model: Optional[str] = None,
        n: Optional[int] = 1,
        quality: Optional[models.Quality] = None,
        response_format: Optional[models.ImageGenerationRequestResponseFormat] = None,
        size: Optional[models.Size] = "1024x1024",
        style: Optional[models.Style] = None,
        user: Optional[str] = None,
        # GPT Image model parameters
        background: Optional[models.ImageGenerationBackground] = None,
        moderation: Optional[models.ImageGenerationModeration] = None,
        output_compression: Optional[int] = None,
        output_format: Optional[models.ImageGenerationOutputFormat] = None,
        partial_images: Optional[int] = None,
        stream: Union[Literal[False], None] = None,
        # Extended parameters for advanced providers
        negative_prompt: Optional[str] = None,
        seed: Optional[int] = None,
        prompt_extend: Optional[bool] = None,
        watermark: Optional[bool] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.ImageGenerationResponse:
        ...

    @overload
    def create(
        self,
        *,
        prompt: str,
        model: Optional[str] = None,
        n: Optional[int] = 1,
        quality: Optional[models.Quality] = None,
        response_format: Optional[models.ImageGenerationRequestResponseFormat] = None,
        size: Optional[models.Size] = "1024x1024",
        style: Optional[models.Style] = None,
        user: Optional[str] = None,
        # GPT Image model parameters
        background: Optional[models.ImageGenerationBackground] = None,
        moderation: Optional[models.ImageGenerationModeration] = None,
        output_compression: Optional[int] = None,
        output_format: Optional[models.ImageGenerationOutputFormat] = None,
        partial_images: Optional[int] = None,
        stream: Literal[True] = ...,
        # Extended parameters for advanced providers
        negative_prompt: Optional[str] = None,
        seed: Optional[int] = None,
        prompt_extend: Optional[bool] = None,
        watermark: Optional[bool] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> eventstreaming.EventStream[models.CreateImageGenerationResponseBody]:
        ...

    def create(
        self,
        *,
        prompt: str,
        model: Optional[str] = None,
        n: Optional[int] = 1,
        quality: Optional[models.Quality] = None,
        response_format: Optional[models.ImageGenerationRequestResponseFormat] = None,
        size: Optional[models.Size] = "1024x1024",
        style: Optional[models.Style] = None,
        user: Optional[str] = None,
        # GPT Image model parameters
        background: Optional[models.ImageGenerationBackground] = None,
        moderation: Optional[models.ImageGenerationModeration] = None,
        output_compression: Optional[int] = None,
        output_format: Optional[models.ImageGenerationOutputFormat] = None,
        partial_images: Optional[int] = None,
        stream: Optional[bool] = None,
        # Extended parameters for advanced providers
        negative_prompt: Optional[str] = None,
        seed: Optional[int] = None,
        prompt_extend: Optional[bool] = None,
        watermark: Optional[bool] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> Union[models.ImageGenerationResponse, eventstreaming.EventStream[models.CreateImageGenerationResponseBody]]:
        r"""Create image

        Generate images from text prompts

        :param prompt: Image description prompt
        :param model: Model name
        :param n: Number of images to generate
        :param quality:
        :param response_format:
        :param size:
        :param style:
        :param user:
        :param background: Background transparency setting (GPT models only)
        :param moderation: Content moderation level (GPT models only)
        :param output_compression: Compression level for webp/jpeg (GPT models only)
        :param output_format: Output format (GPT models only)
        :param partial_images: Number of partial images for streaming (GPT models only)
        :param stream: Enable streaming mode (GPT models only)
        :param negative_prompt: Negative prompt to exclude elements (Qwen, Stability)
        :param seed: Random seed for reproducibility
        :param prompt_extend: Enable AI prompt optimization (Qwen-specific)
        :param watermark: Add watermark to generated images (Qwen-specific)
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.ImageGenerationRequest(
            prompt=prompt,
            model=model,
            n=n,
            quality=quality,
            response_format=response_format,
            size=size,
            style=style,
            user=user,
            background=background,
            moderation=moderation,
            output_compression=output_compression,
            output_format=output_format,
            partial_images=partial_images,
            stream=stream,
            negative_prompt=negative_prompt,
            seed=seed,
            prompt_extend=prompt_extend,
            watermark=watermark,
        )

        req = self._build_request(
            method="POST",
            path="/images/generations",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="text/event-stream" if stream else "application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", models.ImageGenerationRequest
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="createImageGeneration",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "403",
                "422",
                "429",
                "4XX",
                "500",
                "503",
                "5XX",
            ],
            retry_config=retry_config,
            stream=True,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            http_res_text = utils.stream_to_text(http_res)
            return unmarshal_json_response(
                models.ImageGenerationResponse, http_res, http_res_text
            )
        if utils.match_response(http_res, "200", "text/event-stream"):
            return eventstreaming.EventStream(
                http_res,
                lambda raw: utils.unmarshal_json(raw, models.CreateImageGenerationResponseBody),
                sentinel="[DONE]",
                client_ref=self,
            )
        if utils.match_response(http_res, "400", "application/json"):
            http_res_text = utils.stream_to_text(http_res)
            response_data = unmarshal_json_response(
                errors.BadRequestErrorData, http_res, http_res_text
            )
            raise errors.BadRequestError(response_data, http_res, http_res_text)
        if utils.match_response(http_res, "401", "application/json"):
            http_res_text = utils.stream_to_text(http_res)
            response_data = unmarshal_json_response(
                errors.AuthenticationErrorData, http_res, http_res_text
            )
            raise errors.AuthenticationError(response_data, http_res, http_res_text)
        if utils.match_response(http_res, "403", "application/json"):
            http_res_text = utils.stream_to_text(http_res)
            response_data = unmarshal_json_response(
                errors.PermissionDeniedErrorData, http_res, http_res_text
            )
            raise errors.PermissionDeniedError(response_data, http_res, http_res_text)
        if utils.match_response(http_res, "422", "application/json"):
            http_res_text = utils.stream_to_text(http_res)
            response_data = unmarshal_json_response(
                errors.UnprocessableEntityErrorData, http_res, http_res_text
            )
            raise errors.UnprocessableEntityError(response_data, http_res, http_res_text)
        if utils.match_response(http_res, "429", "application/json"):
            http_res_text = utils.stream_to_text(http_res)
            response_data = unmarshal_json_response(errors.RateLimitErrorData, http_res, http_res_text)
            raise errors.RateLimitError(response_data, http_res, http_res_text)
        if utils.match_response(http_res, "500", "application/json"):
            http_res_text = utils.stream_to_text(http_res)
            response_data = unmarshal_json_response(
                errors.InternalServerErrorData, http_res, http_res_text
            )
            raise errors.InternalServerError(response_data, http_res, http_res_text)
        if utils.match_response(http_res, "503", "application/json"):
            http_res_text = utils.stream_to_text(http_res)
            response_data = unmarshal_json_response(
                errors.ServiceUnavailableErrorData, http_res, http_res_text
            )
            raise errors.ServiceUnavailableError(response_data, http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.R9SDefaultError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.R9SDefaultError("API error occurred", http_res, http_res_text)

        raise errors.R9SDefaultError("Unexpected response received", http_res)

    @overload
    async def create_async(
        self,
        *,
        prompt: str,
        model: Optional[str] = None,
        n: Optional[int] = 1,
        quality: Optional[models.Quality] = None,
        response_format: Optional[models.ImageGenerationRequestResponseFormat] = None,
        size: Optional[models.Size] = "1024x1024",
        style: Optional[models.Style] = None,
        user: Optional[str] = None,
        # GPT Image model parameters
        background: Optional[models.ImageGenerationBackground] = None,
        moderation: Optional[models.ImageGenerationModeration] = None,
        output_compression: Optional[int] = None,
        output_format: Optional[models.ImageGenerationOutputFormat] = None,
        partial_images: Optional[int] = None,
        stream: Union[Literal[False], None] = None,
        # Extended parameters for advanced providers
        negative_prompt: Optional[str] = None,
        seed: Optional[int] = None,
        prompt_extend: Optional[bool] = None,
        watermark: Optional[bool] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.ImageGenerationResponse:
        ...

    @overload
    async def create_async(
        self,
        *,
        prompt: str,
        model: Optional[str] = None,
        n: Optional[int] = 1,
        quality: Optional[models.Quality] = None,
        response_format: Optional[models.ImageGenerationRequestResponseFormat] = None,
        size: Optional[models.Size] = "1024x1024",
        style: Optional[models.Style] = None,
        user: Optional[str] = None,
        # GPT Image model parameters
        background: Optional[models.ImageGenerationBackground] = None,
        moderation: Optional[models.ImageGenerationModeration] = None,
        output_compression: Optional[int] = None,
        output_format: Optional[models.ImageGenerationOutputFormat] = None,
        partial_images: Optional[int] = None,
        stream: Literal[True] = ...,
        # Extended parameters for advanced providers
        negative_prompt: Optional[str] = None,
        seed: Optional[int] = None,
        prompt_extend: Optional[bool] = None,
        watermark: Optional[bool] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> eventstreaming.EventStreamAsync[models.CreateImageGenerationResponseBody]:
        ...

    async def create_async(
        self,
        *,
        prompt: str,
        model: Optional[str] = None,
        n: Optional[int] = 1,
        quality: Optional[models.Quality] = None,
        response_format: Optional[models.ImageGenerationRequestResponseFormat] = None,
        size: Optional[models.Size] = "1024x1024",
        style: Optional[models.Style] = None,
        user: Optional[str] = None,
        # GPT Image model parameters
        background: Optional[models.ImageGenerationBackground] = None,
        moderation: Optional[models.ImageGenerationModeration] = None,
        output_compression: Optional[int] = None,
        output_format: Optional[models.ImageGenerationOutputFormat] = None,
        partial_images: Optional[int] = None,
        stream: Optional[bool] = None,
        # Extended parameters for advanced providers
        negative_prompt: Optional[str] = None,
        seed: Optional[int] = None,
        prompt_extend: Optional[bool] = None,
        watermark: Optional[bool] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> Union[models.ImageGenerationResponse, eventstreaming.EventStreamAsync[models.CreateImageGenerationResponseBody]]:
        r"""Create image

        Generate images from text prompts

        :param prompt: Image description prompt
        :param model: Model name
        :param n: Number of images to generate
        :param quality:
        :param response_format:
        :param size:
        :param style:
        :param user:
        :param background: Background transparency setting (GPT models only)
        :param moderation: Content moderation level (GPT models only)
        :param output_compression: Compression level for webp/jpeg (GPT models only)
        :param output_format: Output format (GPT models only)
        :param partial_images: Number of partial images for streaming (GPT models only)
        :param stream: Enable streaming mode (GPT models only)
        :param negative_prompt: Negative prompt to exclude elements (Qwen, Stability)
        :param seed: Random seed for reproducibility
        :param prompt_extend: Enable AI prompt optimization (Qwen-specific)
        :param watermark: Add watermark to generated images (Qwen-specific)
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.ImageGenerationRequest(
            prompt=prompt,
            model=model,
            n=n,
            quality=quality,
            response_format=response_format,
            size=size,
            style=style,
            user=user,
            background=background,
            moderation=moderation,
            output_compression=output_compression,
            output_format=output_format,
            partial_images=partial_images,
            stream=stream,
            negative_prompt=negative_prompt,
            seed=seed,
            prompt_extend=prompt_extend,
            watermark=watermark,
        )

        req = self._build_request_async(
            method="POST",
            path="/images/generations",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="text/event-stream" if stream else "application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", models.ImageGenerationRequest
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="createImageGeneration",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "403",
                "422",
                "429",
                "4XX",
                "500",
                "503",
                "5XX",
            ],
            retry_config=retry_config,
            stream=True,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            http_res_text = await utils.stream_to_text_async(http_res)
            return unmarshal_json_response(
                models.ImageGenerationResponse, http_res, http_res_text
            )
        if utils.match_response(http_res, "200", "text/event-stream"):
            return eventstreaming.EventStreamAsync(
                http_res,
                lambda raw: utils.unmarshal_json(raw, models.CreateImageGenerationResponseBody),
                sentinel="[DONE]",
                client_ref=self,
            )
        if utils.match_response(http_res, "400", "application/json"):
            http_res_text = await utils.stream_to_text_async(http_res)
            response_data = unmarshal_json_response(
                errors.BadRequestErrorData, http_res, http_res_text
            )
            raise errors.BadRequestError(response_data, http_res, http_res_text)
        if utils.match_response(http_res, "401", "application/json"):
            http_res_text = await utils.stream_to_text_async(http_res)
            response_data = unmarshal_json_response(
                errors.AuthenticationErrorData, http_res, http_res_text
            )
            raise errors.AuthenticationError(response_data, http_res, http_res_text)
        if utils.match_response(http_res, "403", "application/json"):
            http_res_text = await utils.stream_to_text_async(http_res)
            response_data = unmarshal_json_response(
                errors.PermissionDeniedErrorData, http_res, http_res_text
            )
            raise errors.PermissionDeniedError(response_data, http_res, http_res_text)
        if utils.match_response(http_res, "422", "application/json"):
            http_res_text = await utils.stream_to_text_async(http_res)
            response_data = unmarshal_json_response(
                errors.UnprocessableEntityErrorData, http_res, http_res_text
            )
            raise errors.UnprocessableEntityError(response_data, http_res, http_res_text)
        if utils.match_response(http_res, "429", "application/json"):
            http_res_text = await utils.stream_to_text_async(http_res)
            response_data = unmarshal_json_response(errors.RateLimitErrorData, http_res, http_res_text)
            raise errors.RateLimitError(response_data, http_res, http_res_text)
        if utils.match_response(http_res, "500", "application/json"):
            http_res_text = await utils.stream_to_text_async(http_res)
            response_data = unmarshal_json_response(
                errors.InternalServerErrorData, http_res, http_res_text
            )
            raise errors.InternalServerError(response_data, http_res, http_res_text)
        if utils.match_response(http_res, "503", "application/json"):
            http_res_text = await utils.stream_to_text_async(http_res)
            response_data = unmarshal_json_response(
                errors.ServiceUnavailableErrorData, http_res, http_res_text
            )
            raise errors.ServiceUnavailableError(response_data, http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.R9SDefaultError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.R9SDefaultError("API error occurred", http_res, http_res_text)

        raise errors.R9SDefaultError("Unexpected response received", http_res)

    @overload
    def edit(
        self,
        *,
        image: Union[
            models.ImageFile,
            models.ImageFileTypedDict,
            list[Union[models.ImageFile, models.ImageFileTypedDict]]
        ],
        prompt: str,
        background: Optional[models.ImageEditBackground] = None,
        input_fidelity: Optional[models.ImageEditInputFidelity] = "low",
        mask: Optional[Union[models.ImageFile, models.ImageFileTypedDict]] = None,
        model: Optional[str] = None,
        moderation: Optional[models.ImageEditModeration] = None,
        n: Optional[int] = 1,
        output_compression: Optional[int] = None,
        output_format: Optional[models.ImageEditOutputFormat] = None,
        partial_images: Optional[int] = 0,
        quality: Optional[models.ImageEditQuality] = None,
        response_format: Optional[models.ImageEditResponseFormat] = None,
        size: Optional[models.ImageEditSize] = "1024x1024",
        stream: Union[Literal[False], None] = None,
        user: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.ImageGenerationResponse:
        ...

    @overload
    def edit(
        self,
        *,
        image: Union[
            models.ImageFile,
            models.ImageFileTypedDict,
            list[Union[models.ImageFile, models.ImageFileTypedDict]]
        ],
        prompt: str,
        background: Optional[models.ImageEditBackground] = None,
        input_fidelity: Optional[models.ImageEditInputFidelity] = "low",
        mask: Optional[Union[models.ImageFile, models.ImageFileTypedDict]] = None,
        model: Optional[str] = None,
        moderation: Optional[models.ImageEditModeration] = None,
        n: Optional[int] = 1,
        output_compression: Optional[int] = None,
        output_format: Optional[models.ImageEditOutputFormat] = None,
        partial_images: Optional[int] = 0,
        quality: Optional[models.ImageEditQuality] = None,
        response_format: Optional[models.ImageEditResponseFormat] = None,
        size: Optional[models.ImageEditSize] = "1024x1024",
        stream: Literal[True] = ...,
        user: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> eventstreaming.EventStream[models.EditImageResponseBody]:
        ...

    def edit(
        self,
        *,
        image: Union[
            models.ImageFile,
            models.ImageFileTypedDict,
            list[Union[models.ImageFile, models.ImageFileTypedDict]]
        ],
        prompt: str,
        background: Optional[models.ImageEditBackground] = None,
        input_fidelity: Optional[models.ImageEditInputFidelity] = "low",
        mask: Optional[Union[models.ImageFile, models.ImageFileTypedDict]] = None,
        model: Optional[str] = None,
        moderation: Optional[models.ImageEditModeration] = None,
        n: Optional[int] = 1,
        output_compression: Optional[int] = None,
        output_format: Optional[models.ImageEditOutputFormat] = None,
        partial_images: Optional[int] = 0,
        quality: Optional[models.ImageEditQuality] = None,
        response_format: Optional[models.ImageEditResponseFormat] = None,
        size: Optional[models.ImageEditSize] = "1024x1024",
        stream: Optional[bool] = None,
        user: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> Union[models.ImageGenerationResponse, eventstreaming.EventStream[models.EditImageResponseBody]]:
        r"""Edit image

        Edit an existing image using a text prompt (inpainting).

        :param image: The image to edit (PNG, <4MB, square)
        :param prompt: Text description of desired edit
        :param model: Model name
        :param mask: Optional mask PNG with transparent edit regions
        :param n: Number of images to generate (1-10)
        :param size: Output size
        :param response_format: 'url' or 'b64_json'
        :param user: End-user identifier
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        # Handle list of images - use first image if list is provided
        image_to_use = image[0] if isinstance(image, list) else image

        request = models.ImageEditRequest(
            image=utils.get_pydantic_model(image_to_use, models.ImageFile),
            prompt=prompt,
            background=background,
            input_fidelity=input_fidelity,
            mask=utils.get_pydantic_model(mask, models.ImageFile) if mask else None,
            model=model,
            moderation=moderation,
            n=n,
            output_compression=output_compression,
            output_format=output_format,
            partial_images=partial_images,
            quality=quality,
            response_format=response_format,
            size=size,
            stream=stream,
            user=user,
        )

        req = self._build_request(
            method="POST",
            path="/images/edits",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="text/event-stream" if stream else "application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "multipart", models.ImageEditRequest
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="editImage",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "403",
                "422",
                "429",
                "4XX",
                "500",
                "503",
                "5XX",
            ],
            retry_config=retry_config,
            stream=True,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            http_res_text = utils.stream_to_text(http_res)
            return unmarshal_json_response(
                models.ImageGenerationResponse, http_res, http_res_text
            )
        if utils.match_response(http_res, "200", "text/event-stream"):
            return eventstreaming.EventStream(
                http_res,
                lambda raw: utils.unmarshal_json(raw, models.EditImageResponseBody),
                sentinel="[DONE]",
                client_ref=self,
            )
        if utils.match_response(http_res, "400", "application/json"):
            http_res_text = utils.stream_to_text(http_res)
            response_data = unmarshal_json_response(
                errors.BadRequestErrorData, http_res, http_res_text
            )
            raise errors.BadRequestError(response_data, http_res, http_res_text)
        if utils.match_response(http_res, "401", "application/json"):
            http_res_text = utils.stream_to_text(http_res)
            response_data = unmarshal_json_response(
                errors.AuthenticationErrorData, http_res, http_res_text
            )
            raise errors.AuthenticationError(response_data, http_res, http_res_text)
        if utils.match_response(http_res, "403", "application/json"):
            http_res_text = utils.stream_to_text(http_res)
            response_data = unmarshal_json_response(
                errors.PermissionDeniedErrorData, http_res, http_res_text
            )
            raise errors.PermissionDeniedError(response_data, http_res, http_res_text)
        if utils.match_response(http_res, "422", "application/json"):
            http_res_text = utils.stream_to_text(http_res)
            response_data = unmarshal_json_response(
                errors.UnprocessableEntityErrorData, http_res, http_res_text
            )
            raise errors.UnprocessableEntityError(response_data, http_res, http_res_text)
        if utils.match_response(http_res, "429", "application/json"):
            http_res_text = utils.stream_to_text(http_res)
            response_data = unmarshal_json_response(errors.RateLimitErrorData, http_res, http_res_text)
            raise errors.RateLimitError(response_data, http_res, http_res_text)
        if utils.match_response(http_res, "500", "application/json"):
            http_res_text = utils.stream_to_text(http_res)
            response_data = unmarshal_json_response(
                errors.InternalServerErrorData, http_res, http_res_text
            )
            raise errors.InternalServerError(response_data, http_res, http_res_text)
        if utils.match_response(http_res, "503", "application/json"):
            http_res_text = utils.stream_to_text(http_res)
            response_data = unmarshal_json_response(
                errors.ServiceUnavailableErrorData, http_res, http_res_text
            )
            raise errors.ServiceUnavailableError(response_data, http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.R9SDefaultError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.R9SDefaultError("API error occurred", http_res, http_res_text)

        raise errors.R9SDefaultError("Unexpected response received", http_res)

    @overload
    async def edit_async(
        self,
        *,
        image: Union[
            models.ImageFile,
            models.ImageFileTypedDict,
            list[Union[models.ImageFile, models.ImageFileTypedDict]]
        ],
        prompt: str,
        background: Optional[models.ImageEditBackground] = None,
        input_fidelity: Optional[models.ImageEditInputFidelity] = "low",
        mask: Optional[Union[models.ImageFile, models.ImageFileTypedDict]] = None,
        model: Optional[str] = None,
        moderation: Optional[models.ImageEditModeration] = None,
        n: Optional[int] = 1,
        output_compression: Optional[int] = None,
        output_format: Optional[models.ImageEditOutputFormat] = None,
        partial_images: Optional[int] = 0,
        quality: Optional[models.ImageEditQuality] = None,
        response_format: Optional[models.ImageEditResponseFormat] = None,
        size: Optional[models.ImageEditSize] = "1024x1024",
        stream: Union[Literal[False], None] = None,
        user: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.ImageGenerationResponse:
        ...

    @overload
    async def edit_async(
        self,
        *,
        image: Union[
            models.ImageFile,
            models.ImageFileTypedDict,
            list[Union[models.ImageFile, models.ImageFileTypedDict]]
        ],
        prompt: str,
        background: Optional[models.ImageEditBackground] = None,
        input_fidelity: Optional[models.ImageEditInputFidelity] = "low",
        mask: Optional[Union[models.ImageFile, models.ImageFileTypedDict]] = None,
        model: Optional[str] = None,
        moderation: Optional[models.ImageEditModeration] = None,
        n: Optional[int] = 1,
        output_compression: Optional[int] = None,
        output_format: Optional[models.ImageEditOutputFormat] = None,
        partial_images: Optional[int] = 0,
        quality: Optional[models.ImageEditQuality] = None,
        response_format: Optional[models.ImageEditResponseFormat] = None,
        size: Optional[models.ImageEditSize] = "1024x1024",
        stream: Literal[True] = ...,
        user: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> eventstreaming.EventStreamAsync[models.EditImageResponseBody]:
        ...

    async def edit_async(
        self,
        *,
        image: Union[
            models.ImageFile,
            models.ImageFileTypedDict,
            list[Union[models.ImageFile, models.ImageFileTypedDict]]
        ],
        prompt: str,
        background: Optional[models.ImageEditBackground] = None,
        input_fidelity: Optional[models.ImageEditInputFidelity] = "low",
        mask: Optional[Union[models.ImageFile, models.ImageFileTypedDict]] = None,
        model: Optional[str] = None,
        moderation: Optional[models.ImageEditModeration] = None,
        n: Optional[int] = 1,
        output_compression: Optional[int] = None,
        output_format: Optional[models.ImageEditOutputFormat] = None,
        partial_images: Optional[int] = 0,
        quality: Optional[models.ImageEditQuality] = None,
        response_format: Optional[models.ImageEditResponseFormat] = None,
        size: Optional[models.ImageEditSize] = "1024x1024",
        stream: Optional[bool] = None,
        user: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> Union[models.ImageGenerationResponse, eventstreaming.EventStreamAsync[models.EditImageResponseBody]]:
        r"""Edit image

        Edit an existing image using a text prompt (inpainting).

        :param image: The image to edit (PNG, <4MB, square)
        :param prompt: Text description of desired edit
        :param model: Model name
        :param mask: Optional mask PNG with transparent edit regions
        :param n: Number of images to generate (1-10)
        :param size: Output size
        :param response_format: 'url' or 'b64_json'
        :param user: End-user identifier
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        # Handle list of images - use first image if list is provided
        image_to_use = image[0] if isinstance(image, list) else image

        request = models.ImageEditRequest(
            image=utils.get_pydantic_model(image_to_use, models.ImageFile),
            prompt=prompt,
            background=background,
            input_fidelity=input_fidelity,
            mask=utils.get_pydantic_model(mask, models.ImageFile) if mask else None,
            model=model,
            moderation=moderation,
            n=n,
            output_compression=output_compression,
            output_format=output_format,
            partial_images=partial_images,
            quality=quality,
            response_format=response_format,
            size=size,
            stream=stream,
            user=user,
        )

        req = self._build_request_async(
            method="POST",
            path="/images/edits",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="text/event-stream" if stream else "application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "multipart", models.ImageEditRequest
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="editImage",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "403",
                "422",
                "429",
                "4XX",
                "500",
                "503",
                "5XX",
            ],
            retry_config=retry_config,
            stream=True,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            http_res_text = await utils.stream_to_text_async(http_res)
            return unmarshal_json_response(
                models.ImageGenerationResponse, http_res, http_res_text
            )
        if utils.match_response(http_res, "200", "text/event-stream"):
            return eventstreaming.EventStreamAsync(
                http_res,
                lambda raw: utils.unmarshal_json(raw, models.EditImageResponseBody),
                sentinel="[DONE]",
                client_ref=self,
            )
        if utils.match_response(http_res, "400", "application/json"):
            http_res_text = await utils.stream_to_text_async(http_res)
            response_data = unmarshal_json_response(
                errors.BadRequestErrorData, http_res, http_res_text
            )
            raise errors.BadRequestError(response_data, http_res, http_res_text)
        if utils.match_response(http_res, "401", "application/json"):
            http_res_text = await utils.stream_to_text_async(http_res)
            response_data = unmarshal_json_response(
                errors.AuthenticationErrorData, http_res, http_res_text
            )
            raise errors.AuthenticationError(response_data, http_res, http_res_text)
        if utils.match_response(http_res, "403", "application/json"):
            http_res_text = await utils.stream_to_text_async(http_res)
            response_data = unmarshal_json_response(
                errors.PermissionDeniedErrorData, http_res, http_res_text
            )
            raise errors.PermissionDeniedError(response_data, http_res, http_res_text)
        if utils.match_response(http_res, "422", "application/json"):
            http_res_text = await utils.stream_to_text_async(http_res)
            response_data = unmarshal_json_response(
                errors.UnprocessableEntityErrorData, http_res, http_res_text
            )
            raise errors.UnprocessableEntityError(response_data, http_res, http_res_text)
        if utils.match_response(http_res, "429", "application/json"):
            http_res_text = await utils.stream_to_text_async(http_res)
            response_data = unmarshal_json_response(errors.RateLimitErrorData, http_res, http_res_text)
            raise errors.RateLimitError(response_data, http_res, http_res_text)
        if utils.match_response(http_res, "500", "application/json"):
            http_res_text = await utils.stream_to_text_async(http_res)
            response_data = unmarshal_json_response(
                errors.InternalServerErrorData, http_res, http_res_text
            )
            raise errors.InternalServerError(response_data, http_res, http_res_text)
        if utils.match_response(http_res, "503", "application/json"):
            http_res_text = await utils.stream_to_text_async(http_res)
            response_data = unmarshal_json_response(
                errors.ServiceUnavailableErrorData, http_res, http_res_text
            )
            raise errors.ServiceUnavailableError(response_data, http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.R9SDefaultError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.R9SDefaultError("API error occurred", http_res, http_res_text)

        raise errors.R9SDefaultError("Unexpected response received", http_res)



from __future__ import annotations

import json
from dataclasses import dataclass
from pathlib import Path
from typing import Any, Dict, List, Optional, Sequence

try:
    import tomllib  # pyright: ignore[reportMissingImports]
except Exception:  # pragma: no cover
    tomllib = None  # type: ignore[assignment]

try:
    import tomli  # pyright: ignore[reportMissingImports]
except Exception:  # pragma: no cover
    tomli = None  # type: ignore[assignment]


@dataclass
class BotConfig:
    name: str
    description: Optional[str] = None
    system_prompt: Optional[str] = None
    temperature: Optional[float] = None
    top_p: Optional[float] = None
    max_tokens: Optional[int] = None
    presence_penalty: Optional[float] = None
    frequency_penalty: Optional[float] = None


def bots_root() -> Path:
    return Path.home() / ".r9s" / "bots"


def bot_path(name: str) -> Path:
    safe = name.strip()
    if not safe:
        raise ValueError("bot name cannot be empty")
    return bots_root() / f"{safe}.toml"


def _load_toml(path: Path) -> Dict[str, Any]:
    raw = path.read_bytes()
    if tomllib is not None:
        data = tomllib.loads(raw.decode("utf-8"))
    elif tomli is not None:
        data = tomli.loads(raw.decode("utf-8"))
    else:
        raise RuntimeError("TOML parser is not available (need tomllib or tomli)")
    if not isinstance(data, dict):
        raise ValueError(f"invalid bot config: {path}")
    return data


def _toml_quote(value: str) -> str:
    return json.dumps(value, ensure_ascii=False)


def _toml_multiline(value: str) -> str:
    # Prefer literal multiline for readability; fall back when delimiter conflicts.
    if "'''" not in value:
        return "'''\n" + value + "\n'''"
    if '"""' not in value:
        return '"""\n' + value + '\n"""'
    return _toml_quote(value)


def _toml_format_array(items: Sequence[str]) -> str:
    return json.dumps(list(items), ensure_ascii=False)


def _dump_bot_toml(bot: BotConfig) -> str:
    lines: List[str] = []
    if bot.description:
        lines.append(f"description = {_toml_quote(bot.description)}")
    if bot.system_prompt is not None:
        lines.append(f"system_prompt = {_toml_multiline(bot.system_prompt)}")
    if bot.temperature is not None:
        lines.append(f"temperature = {bot.temperature}")
    if bot.top_p is not None:
        lines.append(f"top_p = {bot.top_p}")
    if bot.max_tokens is not None:
        lines.append(f"max_tokens = {bot.max_tokens}")
    if bot.presence_penalty is not None:
        lines.append(f"presence_penalty = {bot.presence_penalty}")
    if bot.frequency_penalty is not None:
        lines.append(f"frequency_penalty = {bot.frequency_penalty}")
    return "\n".join(lines).rstrip() + "\n"


def save_bot(bot: BotConfig) -> Path:
    root = bots_root()
    root.mkdir(parents=True, exist_ok=True)
    path = bot_path(bot.name)
    path.write_text(_dump_bot_toml(bot), encoding="utf-8")
    return path


def load_bot(name: str) -> BotConfig:
    path = bot_path(name)
    data = _load_toml(path)

    description_raw = data.get("description")
    description = (
        description_raw.strip()
        if isinstance(description_raw, str) and description_raw.strip()
        else None
    )

    system_prompt = (
        str(data["system_prompt"])
        if isinstance(data.get("system_prompt"), str)
        else None
    )
    system_prompt = (
        system_prompt.strip() if (system_prompt and system_prompt.strip()) else None
    )

    temperature = data.get("temperature")
    temperature = (
        float(temperature)
        if isinstance(temperature, (int, float)) and temperature is not None
        else None
    )

    top_p = data.get("top_p")
    top_p = (
        float(top_p) if isinstance(top_p, (int, float)) and top_p is not None else None
    )

    max_tokens = data.get("max_tokens")
    max_tokens = int(max_tokens) if isinstance(max_tokens, int) else None

    presence_penalty = data.get("presence_penalty")
    presence_penalty = (
        float(presence_penalty)
        if isinstance(presence_penalty, (int, float)) and presence_penalty is not None
        else None
    )

    frequency_penalty = data.get("frequency_penalty")
    frequency_penalty = (
        float(frequency_penalty)
        if isinstance(frequency_penalty, (int, float)) and frequency_penalty is not None
        else None
    )

    return BotConfig(
        name=name.strip(),
        description=description,
        system_prompt=system_prompt,
        temperature=temperature,
        top_p=top_p,
        max_tokens=max_tokens,
        presence_penalty=presence_penalty,
        frequency_penalty=frequency_penalty,
    )


def list_bots() -> List[str]:
    root = bots_root()
    if not root.exists():
        return []
    out: List[str] = []
    for p in sorted(root.glob("*.toml")):
        out.append(p.stem)
    return out


def delete_bot(name: str) -> Path:
    path = bot_path(name)
    path.unlink()
    return path



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
import pydantic
from pydantic.functional_validators import AfterValidator
from r9s.types import BaseModel
from r9s.utils import validate_const
from typing import Literal
from typing_extensions import Annotated, TypedDict


class AnthropicPingTypedDict(TypedDict):
    r"""SSE heartbeat event to keep connection alive"""

    type: Literal["ping"]


class AnthropicPing(BaseModel):
    r"""SSE heartbeat event to keep connection alive"""

    TYPE: Annotated[
        Annotated[Literal["ping"], AfterValidator(validate_const("ping"))],
        pydantic.Field(alias="type"),
    ] = "ping"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
import httpx
from typing import Union
from typing_extensions import TypeAliasType


CreateAudioSpeechResponseTypedDict = TypeAliasType(
    "CreateAudioSpeechResponseTypedDict",
    Union[
        httpx.Response,
        httpx.Response,
        httpx.Response,
        httpx.Response,
        httpx.Response,
        httpx.Response,
    ],
)


CreateAudioSpeechResponse = TypeAliasType(
    "CreateAudioSpeechResponse",
    Union[
        httpx.Response,
        httpx.Response,
        httpx.Response,
        httpx.Response,
        httpx.Response,
        httpx.Response,
    ],
)



from __future__ import annotations

import pytest

from r9s.skills.exceptions import InvalidSkillError
from r9s.skills.parser import parse_skill_markdown


def test_parse_skill_markdown() -> None:
    content = """---
name: code-review
description: Review code
allowed-tools: git bash
metadata:
  author: r9s
---

# Code Review

Use this skill.
"""
    metadata, body = parse_skill_markdown(content)
    assert metadata.name == "code-review"
    assert metadata.description == "Review code"
    assert metadata.allowed_tools == ["git", "bash"]
    assert metadata.metadata == {"author": "r9s"}
    assert body.startswith("# Code Review")


def test_parse_missing_frontmatter() -> None:
    content = "# No frontmatter here"
    with pytest.raises(InvalidSkillError, match="Missing YAML frontmatter"):
        parse_skill_markdown(content)


def test_parse_unterminated_frontmatter() -> None:
    content = """---
name: broken
description: Never closed
"""
    with pytest.raises(InvalidSkillError, match="Unterminated YAML frontmatter"):
        parse_skill_markdown(content)


def test_parse_empty_file() -> None:
    with pytest.raises(InvalidSkillError, match="empty"):
        parse_skill_markdown("")
    with pytest.raises(InvalidSkillError, match="empty"):
        parse_skill_markdown("   \n\n   ")


def test_parse_invalid_yaml() -> None:
    content = """---
name: [unclosed
description: bad yaml
---

body
"""
    with pytest.raises(InvalidSkillError, match="Invalid YAML"):
        parse_skill_markdown(content)


def test_parse_missing_required_fields() -> None:
    # Missing name
    content = """---
description: Has description but no name
---

body
"""
    with pytest.raises(InvalidSkillError, match="name is required"):
        parse_skill_markdown(content)

    # Missing description
    content = """---
name: has-name
---

body
"""
    with pytest.raises(InvalidSkillError, match="description is required"):
        parse_skill_markdown(content)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from r9s.types import BaseModel
from typing_extensions import TypedDict


class EditChoiceTypedDict(TypedDict):
    text: str
    index: int


class EditChoice(BaseModel):
    text: str

    index: int



from __future__ import annotations

import argparse
from dataclasses import dataclass
import json
import os
import shutil
import stat
import subprocess
import sys
import tarfile
import tempfile
import urllib.request
import zipfile
from datetime import datetime, timezone
from difflib import unified_diff
from pathlib import Path
from typing import Dict, Optional

from r9s.agents.exceptions import AgentExistsError, AgentNotFoundError
from r9s.agents.local_store import (
    LocalAgentStore,
    LocalAuditStore,
    agent_path,
    delete_agent,
    list_agents,
    load_agent,
    load_version,
    read_agent_name_from_manifest,
    save_agent,
    save_version,
)
from r9s.agents.models import AgentStatus
from r9s.cli_tools.bots import load_bot
from r9s.cli_tools.ui.rich_output import print_markdown
from r9s.cli_tools.ui.terminal import (
    FG_RED,
    FG_YELLOW,
    error,
    header,
    info,
    prompt_text,
    success,
    warning,
)


def _require_name(name: Optional[str]) -> str:
    if name and name.strip():
        return name.strip()
    raise SystemExit("agent name is required")


def _is_interactive() -> bool:
    return sys.stdin.isatty()


def _prompt_multiline_required(message: str) -> str:
    header(message)
    first = prompt_text("> ", color=FG_YELLOW)
    while not first:
        first = prompt_text("> ", color=FG_RED)
    lines = [first]
    while True:
        line = prompt_text("> ", color=FG_YELLOW)
        if not line:
            break
        lines.append(line)
    out = "\n".join(lines).rstrip()
    if not out:
        raise SystemExit("instructions cannot be empty")
    return out


def _resolve_model(model: Optional[str]) -> str:
    """Resolve model from argument, environment, or interactive prompt."""
    if model and model.strip():
        return model.strip()
    # Fall back to R9S_MODEL environment variable
    env_model = os.environ.get("R9S_MODEL", "").strip()
    if env_model:
        return env_model
    # Interactive prompt as last resort
    if _is_interactive():
        model = prompt_text("Model (or set R9S_MODEL): ")
        if model:
            return model
    raise SystemExit("model is required (use --model or set R9S_MODEL)")


def _parse_params(param_text: Optional[str]) -> Dict[str, object]:
    if not param_text:
        return {}
    try:
        data = json.loads(param_text)
    except json.JSONDecodeError as exc:
        raise SystemExit(f"Invalid params JSON: {exc}") from exc
    if not isinstance(data, dict):
        raise SystemExit("params must be a JSON object")
    return data


def _read_instructions_file(file_path: str) -> str:
    """Read instructions from a file."""
    path = Path(file_path).expanduser()
    if not path.exists():
        raise SystemExit(f"File not found: {file_path}")
    try:
        content = path.read_text(encoding="utf-8").strip()
    except OSError as exc:
        raise SystemExit(f"Failed to read file: {exc}") from exc
    if not content:
        raise SystemExit("Instructions file is empty")
    return content


def _edit_in_editor(initial_text: str = "", agent_name: str = "") -> str:
    """Open $EDITOR for editing instructions (like git commit)."""
    editor = os.environ.get("VISUAL") or os.environ.get("EDITOR", "vi")

    # Prepare initial content with help comments
    help_text = """
# Enter instructions for the agent above.
# Lines starting with '#' will be ignored.
# Save and close the editor to continue.
# Leave empty to abort.
"""
    if agent_name:
        help_text = f"# Agent: {agent_name}\n" + help_text

    content = initial_text + "\n" + help_text if initial_text else help_text.lstrip()

    with tempfile.NamedTemporaryFile(
        mode="w", suffix=".md", delete=False, encoding="utf-8"
    ) as f:
        f.write(content)
        tmp_path = f.name

    try:
        result = subprocess.run([editor, tmp_path], check=False)
        if result.returncode != 0:
            raise SystemExit(f"Editor exited with code {result.returncode}")

        with open(tmp_path, encoding="utf-8") as f:
            lines = f.readlines()

        # Filter out comment lines
        filtered = [line for line in lines if not line.lstrip().startswith("#")]
        output = "".join(filtered).strip()

        if not output:
            raise SystemExit("Instructions cannot be empty (aborted)")

        return output
    finally:
        try:
            os.unlink(tmp_path)
        except OSError:
            pass


def _get_instructions(args: argparse.Namespace, existing: str = "") -> str:
    """Get instructions from various sources with priority:

    1. --instructions-file (highest) - explicit file path
    2. --edit - opens $EDITOR
    3. --instructions @filename - file reference (like curl, gh)
       - @- reads from stdin
       - @@ escapes literal @ (e.g., "@@mention" -> "@mention")
    4. --instructions "text" - inline text
    5. Interactive prompt (if TTY)
    """
    # Priority 1: Explicit file flag
    if getattr(args, "instructions_file", None):
        return _read_instructions_file(args.instructions_file)

    # Priority 2: Editor
    if getattr(args, "edit", False):
        return _edit_in_editor(existing, getattr(args, "name", ""))

    # Priority 3 & 4: Inline or @filename convention
    instr = getattr(args, "instructions", None)
    if instr is not None:
        text = instr.strip()
        if text:
            if text.startswith("@"):
                # @@ escape sequence: "@@text" -> "@text"
                if text.startswith("@@"):
                    return text[1:]
                # @filename convention (like curl, gh)
                filename = text[1:].strip()
                if not filename:
                    raise SystemExit(
                        "Invalid --instructions: '@' must be followed by a filename (or @- for stdin)"
                    )
                # @- reads from stdin
                if filename == "-":
                    content = sys.stdin.read().strip()
                    if not content:
                        raise SystemExit("No instructions provided on stdin")
                    return content
                return _read_instructions_file(filename)
            return text

    # Priority 5: Interactive
    if _is_interactive():
        info("Enter instructions for this agent.")
        return _prompt_multiline_required("Instructions (end with empty line):")

    raise SystemExit(
        "Missing instructions. Use --instructions, --instructions-file, or --edit"
    )


def _is_within(path: Path, root: Path) -> bool:
    try:
        path.resolve().relative_to(root.resolve())
    except ValueError:
        return False
    return True


def _is_git_ref(ref: str) -> bool:
    if ref.startswith("github:"):
        return True
    if ref.startswith("git@") or ref.startswith("ssh://"):
        return True
    return ref.endswith(".git")


@dataclass
class GitHubRef:
    """Parsed GitHub reference with optional path and branch."""

    owner: str
    repo: str
    path: Optional[str] = None
    branch: str = "main"


def _parse_github_ref(ref: str) -> GitHubRef:
    """Parse github:owner/repo[/path][@branch] format.

    Supports:
    - github:owner/repo
    - github:owner/repo/path/to/agent
    - github:owner/repo/path/to/agent@branch
    - github:owner/repo@branch
    """
    if not ref.startswith("github:"):
        raise ValueError("Not a github: reference")

    rest = ref[7:].strip("/")  # Remove "github:" prefix

    # Check for @branch suffix
    branch = "main"
    if "@" in rest:
        rest, branch = rest.rsplit("@", 1)

    parts = rest.split("/")
    if len(parts) < 2:
        raise SystemExit("Invalid GitHub reference (expected github:owner/repo)")

    owner = parts[0]
    repo = parts[1]
    path = "/".join(parts[2:]) if len(parts) > 2 else None

    return GitHubRef(owner=owner, repo=repo, path=path, branch=branch)


def _normalize_git_ref(ref: str) -> str:
    if not ref.startswith("github:"):
        return ref
    parsed = _parse_github_ref(ref)
    return f"https://github.com/{parsed.owner}/{parsed.repo}.git"


def _clone_repo(ref: str, dest: Path) -> None:
    repo = _normalize_git_ref(ref)
    result = subprocess.run(
        ["git", "clone", "--depth", "1", repo, str(dest)],
        check=False,
        stdout=subprocess.PIPE,
        stderr=subprocess.PIPE,
        text=True,
    )
    if result.returncode != 0:
        stderr = result.stderr.strip() or result.stdout.strip()
        raise SystemExit(f"git clone failed: {stderr or 'unknown error'}")


def _download_archive(url: str, dest: Path, max_bytes: int = 50 * 1024 * 1024) -> Path:
    request = urllib.request.Request(url, headers={"User-Agent": "r9s-agent-pull"})
    with urllib.request.urlopen(request) as response:
        size = response.headers.get("Content-Length")
        if size and int(size) > max_bytes:
            raise SystemExit("Archive too large to download")
        out_path = dest / Path(url).name
        total = 0
        with open(out_path, "wb") as handle:
            while True:
                chunk = response.read(1024 * 1024)
                if not chunk:
                    break
                total += len(chunk)
                if total > max_bytes:
                    raise SystemExit("Archive exceeds size limit")
                handle.write(chunk)
    return out_path


def _safe_extract_zip(archive: Path, dest: Path) -> None:
    with zipfile.ZipFile(archive) as zf:
        for member in zf.infolist():
            if stat.S_IFMT(member.external_attr >> 16) == stat.S_IFLNK:
                raise SystemExit("Archive contains symlinks")
            target = dest / member.filename
            if not _is_within(target, dest):
                raise SystemExit("Archive contains invalid paths")
            if member.is_dir():
                target.mkdir(parents=True, exist_ok=True)
            else:
                target.parent.mkdir(parents=True, exist_ok=True)
                with zf.open(member) as src, open(target, "wb") as dst:
                    shutil.copyfileobj(src, dst)


def _safe_extract_tar(archive: Path, dest: Path) -> None:
    with tarfile.open(archive) as tf:
        for member in tf.getmembers():
            target = dest / member.name
            if not _is_within(target, dest):
                raise SystemExit("Archive contains invalid paths")
            if member.issym() or member.islnk():
                raise SystemExit("Archive contains symlinks")
            if not (member.isdir() or member.isreg()):
                raise SystemExit("Archive contains unsupported entries")
            if member.isdir():
                target.mkdir(parents=True, exist_ok=True)
            else:
                target.parent.mkdir(parents=True, exist_ok=True)
                extracted = tf.extractfile(member)
                if extracted is not None:
                    with open(target, "wb") as dst:
                        shutil.copyfileobj(extracted, dst)


def _resolve_bundle_path(root: Path, path: Optional[str]) -> Path:
    base = root
    if path:
        base = (root / path).resolve()
    if not base.exists():
        raise SystemExit("Bundle path not found")
    if not base.is_dir():
        raise SystemExit("Bundle path must be a directory")
    if not _is_within(base, root):
        raise SystemExit("Bundle path escapes repository")
    return base


def _copy_agent_bundle(src: Path, dest: Path, name_override: Optional[str]) -> str:
    manifest = src / "agent.toml"
    if not manifest.exists() or not manifest.is_file() or manifest.is_symlink():
        raise SystemExit("agent.toml not found in bundle")
    versions_dir = src / "versions"
    if (
        not versions_dir.exists()
        or not versions_dir.is_dir()
        or versions_dir.is_symlink()
    ):
        raise SystemExit("versions/ directory not found in bundle")

    agent_name = read_agent_name_from_manifest(manifest)
    final_name = name_override.strip() if name_override else agent_name
    if not final_name:
        raise SystemExit("agent name cannot be empty")

    dest.mkdir(parents=True, exist_ok=True)
    dest_manifest = dest / "agent.toml"
    shutil.copy2(manifest, dest_manifest)

    dest_versions = dest / "versions"
    dest_versions.mkdir(parents=True, exist_ok=True)
    for entry in versions_dir.iterdir():
        if entry.is_symlink():
            raise SystemExit("Bundle contains symlinks")
        if entry.is_file() and entry.suffix == ".toml":
            shutil.copy2(entry, dest_versions / entry.name)

    if final_name != agent_name:
        agent_data = load_agent(final_name)
        agent_data.name = final_name
        save_agent(agent_data)

    return final_name


def _cleanup_agent_dir(name: str) -> None:
    try:
        delete_agent(name)
    except Exception:
        pass


def handle_agent_list(_: argparse.Namespace) -> None:
    names = list_agents()
    if not names:
        info("No agents found.")
        return
    header("Agents")
    for name in names:
        try:
            agent = load_agent(name)
        except AgentNotFoundError:
            continue
        print(f"- {agent.name} (current: {agent.current_version})")


def handle_agent_show(args: argparse.Namespace) -> None:
    name = _require_name(args.name)
    agent = load_agent(name)
    header(f"Agent: {agent.name}")
    print(f"- id: {agent.id}")
    if agent.description:
        print(f"- description: {agent.description}")
    print(f"- current_version: {agent.current_version}")
    print(f"- created_at: {agent.created_at.isoformat()}")
    print(f"- updated_at: {agent.updated_at.isoformat()}")
    version = load_version(name, agent.current_version)
    print(f"- model: {version.model}")
    print(f"- provider: {version.provider}")
    print(f"- status: {version.status.value}")
    if version.variables:
        print(f"- variables: {', '.join(version.variables)}")
    if version.skills:
        print(f"- skills: {', '.join(version.skills)}")
    if getattr(args, "instructions", False):
        print()
        header("Instructions")
        print_markdown(version.instructions)


def handle_agent_create(args: argparse.Namespace) -> None:
    name = _require_name(args.name)
    instructions = _get_instructions(args)
    model = _resolve_model(args.model)
    provider = (args.provider or "r9s").strip()
    description = (args.description or "").strip()
    params = _parse_params(args.params)
    skills = getattr(args, "skill", []) or []

    store = LocalAgentStore()
    try:
        agent = store.create(
            name,
            instructions=instructions,
            model=model,
            provider=provider,
            description=description,
            change_reason=args.reason or "",
            model_params=params,
            skills=skills,
        )
    except AgentExistsError:
        raise SystemExit(
            f"Agent '{name}' already exists. Use 'r9s agent update {name}' to modify it."
        )
    success(f"Created agent: {agent.name} (version {agent.current_version})")


def handle_agent_update(args: argparse.Namespace) -> None:
    name = _require_name(args.name)

    # Load existing instructions for editor pre-population
    existing_instructions = ""
    if getattr(args, "edit", False):
        try:
            agent = load_agent(name)
            version = load_version(name, agent.current_version)
            existing_instructions = version.instructions
        except AgentNotFoundError:
            pass

    instructions = _get_instructions(args, existing=existing_instructions)
    params = _parse_params(args.params)

    store = LocalAgentStore()
    try:
        update_kwargs = {
            "instructions": instructions,
            "model": args.model,
            "provider": args.provider,
            "change_reason": args.reason or "",
            "bump": args.bump,
        }
        if params:
            update_kwargs["model_params"] = params
        # Only update skills if --skill was explicitly provided
        skills = getattr(args, "skill", None)
        if skills is not None:
            update_kwargs["skills"] = skills
        version = store.update(
            name,
            **update_kwargs,
        )
    except AgentNotFoundError as exc:
        raise SystemExit(str(exc)) from exc
    success(f"Updated agent: {name} -> {version.version}")


def handle_agent_delete(args: argparse.Namespace) -> None:
    name = _require_name(args.name)
    confirm = prompt_text(f"Delete agent '{name}'? [y/N]: ", color=FG_RED).lower()
    if confirm not in ("y", "yes"):
        warning("Cancelled.")
        return
    try:
        path = delete_agent(name)
    except AgentNotFoundError:
        error("Agent not found.")
        return
    success(f"Deleted: {path}")


def handle_agent_history(args: argparse.Namespace) -> None:
    name = _require_name(args.name)
    store = LocalAgentStore()
    versions = store.list_versions(name)
    if not versions:
        info("No versions found.")
        return
    header(f"Agent history: {name}")
    for version in versions:
        print(
            f"- {version.version} ({version.status.value})"
            + (f" parent={version.parent_version}" if version.parent_version else "")
        )


def handle_agent_diff(args: argparse.Namespace) -> None:
    name = _require_name(args.name)
    v1 = load_version(name, args.v1)
    v2 = load_version(name, args.v2)
    header(f"Diff: {name} {v1.version} -> {v2.version}")
    diff = unified_diff(
        v1.instructions.splitlines(),
        v2.instructions.splitlines(),
        fromfile=v1.version,
        tofile=v2.version,
        lineterm="",
    )
    print("\n".join(diff))


def handle_agent_rollback(args: argparse.Namespace) -> None:
    name = _require_name(args.name)
    version = args.version
    agent = load_agent(name)
    load_version(name, version)
    agent.current_version = version
    agent.updated_at = datetime.now(timezone.utc).replace(microsecond=0)
    save_agent(agent)
    success(f"Rolled back {name} to {version}")


def _update_status(name: str, version: str, status: AgentStatus) -> None:
    agent_version = load_version(name, version)
    agent_version.status = status
    save_version(name, agent_version)


def handle_agent_approve(args: argparse.Namespace) -> None:
    name = _require_name(args.name)
    _update_status(name, args.version, AgentStatus.APPROVED)
    success(f"Approved {name} {args.version}")


def handle_agent_deprecate(args: argparse.Namespace) -> None:
    name = _require_name(args.name)
    _update_status(name, args.version, AgentStatus.DEPRECATED)
    success(f"Deprecated {name} {args.version}")


def handle_agent_audit(args: argparse.Namespace) -> None:
    store = LocalAuditStore()
    entries = store.query(
        agent=_require_name(args.name),
        request_id=args.request_id,
        last=args.last,
    )
    if not entries:
        info("No audit entries found.")
        return
    header(f"Audit: {args.name}")
    for entry in entries:
        print(
            f"- {entry.timestamp.isoformat()} {entry.request_id}"
            f" {entry.agent_version} input={entry.input_tokens} output={entry.output_tokens}"
        )


def handle_agent_export(args: argparse.Namespace) -> None:
    name = _require_name(args.name)
    agent = load_agent(name)
    versions = LocalAgentStore().list_versions(name)
    payload = {
        "agent": {
            "id": agent.id,
            "name": agent.name,
            "description": agent.description,
            "current_version": agent.current_version,
            "created_at": agent.created_at.isoformat(),
            "updated_at": agent.updated_at.isoformat(),
        },
        "versions": [
            {
                "version": v.version,
                "content_hash": v.content_hash,
                "instructions": v.instructions,
                "model": v.model,
                "provider": v.provider,
                "tools": v.tools,
                "files": v.files,
                "variables": v.variables,
                "model_params": v.model_params,
                "created_at": v.created_at.isoformat(),
                "created_by": v.created_by,
                "change_reason": v.change_reason,
                "status": v.status.value,
                "parent_version": v.parent_version,
            }
            for v in versions
        ],
    }
    print(json.dumps(payload, ensure_ascii=False, indent=2))


def handle_agent_import_bot(args: argparse.Namespace) -> None:
    name = _require_name(args.name)
    model = _resolve_model(args.model)
    bot = load_bot(name)
    if not bot.system_prompt:
        raise SystemExit("Bot has no system_prompt to import.")
    store = LocalAgentStore()
    store.create(
        name,
        instructions=bot.system_prompt,
        model=model,
        provider=args.provider or "r9s",
        description=bot.description or "",
        change_reason="Imported from bot",
        model_params={
            "temperature": bot.temperature,
            "top_p": bot.top_p,
            "max_tokens": bot.max_tokens,
            "presence_penalty": bot.presence_penalty,
            "frequency_penalty": bot.frequency_penalty,
        },
    )
    success(f"Imported bot '{name}' as agent.")


def handle_agent_pull(args: argparse.Namespace) -> None:
    ref = args.ref.strip()
    name_override = (args.name or "").strip() or None
    path = args.path
    force = getattr(args, "force", False)

    # Extract path from github:owner/repo/path format if present
    if ref.startswith("github:") and not path:
        try:
            parsed = _parse_github_ref(ref)
            if parsed.path:
                path = parsed.path
        except (ValueError, SystemExit):
            pass  # Fall through to existing error handling

    with tempfile.TemporaryDirectory() as temp_dir:
        temp_root = Path(temp_dir)
        if _is_git_ref(ref):
            repo_dir = temp_root / "repo"
            _clone_repo(ref, repo_dir)
            bundle_root = _resolve_bundle_path(repo_dir, path)
        else:
            local_path = Path(ref).expanduser()
            if local_path.exists():
                bundle_root = _resolve_bundle_path(local_path, path)
            elif ref.startswith("http://") or ref.startswith("https://"):
                try:
                    archive = _download_archive(ref, temp_root)
                except Exception as exc:
                    raise SystemExit(f"Failed to download archive: {exc}") from exc
                extract_dir = temp_root / "extract"
                extract_dir.mkdir(parents=True, exist_ok=True)
                if archive.suffix == ".zip":
                    _safe_extract_zip(archive, extract_dir)
                elif archive.name.endswith((".tar.gz", ".tgz")):
                    _safe_extract_tar(archive, extract_dir)
                else:
                    raise SystemExit("Unsupported archive format (use .zip or .tar.gz)")
                bundle_root = _resolve_bundle_path(extract_dir, path)
            else:
                raise SystemExit("Unsupported source (use git ref, local path, or HTTP archive)")

        manifest = bundle_root / "agent.toml"
        agent_name = read_agent_name_from_manifest(manifest)
        final_name = name_override or agent_name
        dest = agent_path(final_name)

        if dest.exists():
            if not force:
                raise SystemExit(f"Agent already exists: {final_name} (use --force)")
            _cleanup_agent_dir(final_name)

        final_name = _copy_agent_bundle(bundle_root, dest, name_override)
        try:
            agent = load_agent(final_name)
            load_version(final_name, agent.current_version)
            for version_file in (dest / "versions").glob("*.toml"):
                load_version(final_name, version_file.stem)
        except Exception as exc:
            _cleanup_agent_dir(final_name)
            raise SystemExit(f"Invalid agent bundle: {exc}") from exc

    success(f"Pulled agent '{final_name}'")



from __future__ import annotations

from pathlib import Path

from r9s.cli_tools.chat_cli import (
    SessionMeta,
    SessionRecord,
    _history_root,
    _list_sessions,
    _save_history,
)


def test_resume_list_display_is_time_and_prompt_only(temp_home: Path) -> None:  # noqa: ARG001
    root = _history_root()
    path = root / "demo.json"
    record = SessionRecord(
        meta=SessionMeta(
            session_id="demo",
            created_at="2025-12-27T07:28:52+00:00",
            updated_at="2025-12-27T07:28:52+00:00",
            base_url="https://api.example.com/v1",
            model="gpt-5-mini",
            system_prompt=None,
        ),
        messages=[
            {"role": "user", "content": "generate\\ndiff\\nhello"},
        ],
    )
    _save_history(str(path), record)

    sessions = _list_sessions(root)
    assert sessions
    display = sessions[0].display
    assert "2025-" in display
    assert "generate" in display
    assert "\\n" not in display
    assert "http" not in display
    assert "gpt" not in display
    assert ".json" not in display
